/**********************************************************************/
/* The Minstrel's Song is (c) 1998 by Jonathan Hayward.  See the file */
/* lib/copyrigh.txt for a copyright and license statement.            */
/**********************************************************************/

#include <sys/types.h>
#include <curses.h>
#include <stdio.h>
#include <stdlib.h>
#include "tms.h"

struct clist *master_clist;
struct small_creature *player;
struct ilist *master_ilist;
struct game_variables_struct *game_variables;
struct level *current_level, *countryside, *main_city, *prev_level;
struct llist *master_llist;
struct master_item *master_items[NUM_ITEMS][3];
struct pointer_struct *master_pointers;
WINDOW *top_text, *left_panel, *bottom_panel, *dungeon, *puzzle;
long armor[NUM_DIST_SLOTS][3];
long device[NUM_DIST_SLOTS][3];
long furniture[NUM_DIST_SLOTS][3];
long liquid[NUM_DIST_SLOTS][3];
long natural[NUM_DIST_SLOTS][3];
long non_device_equipment[NUM_DIST_SLOTS][3];
long non_device_treasure[NUM_DIST_SLOTS][3];
long num_master_creatures;
long supplies[NUM_DIST_SLOTS][3];
long weapon[NUM_DIST_SLOTS][3];
long building_item[NUM_DIST_SLOTS][3];
long cavern_item[NUM_DIST_SLOTS][3];
long outdoors_item[NUM_DIST_SLOTS][3];
long tiny_item[NUM_DIST_SLOTS][3];

void init(void)
    {
    long i;
	printf("Reached 1\n");
    initscr();
	printf("Reached 2\n");
    tsrandom(); /* tsrandom in machdep.c */
    master_clist = NULL;
    master_ilist = NULL;
    master_llist = NULL;
    game_variables=(struct game_variables_struct *) 
      calloc(1, sizeof(struct game_variables_struct));
#ifdef DEBUG
    game_variables->me = game_variables;
#endif
    game_variables->elapsed_units=(DAY/2);
    game_variables->elapsed_days=(1000 + (ltrandom() % 4000)) * YEAR +
      (ltrandom() % YEAR);
    game_variables->flags=DURING_INIT|DAYLIGHT|TWILIGHT|MAX_REFRESH|
      MESSAGES_PROMPT;
    game_variables->indent_width = INDENT_DEFAULT;
    game_variables->messages_printed = 0;
    game_variables->fs_messages_printed = 0;
    game_variables->quests_completed = 0;
    game_variables->current_quest = SENT_ON_QUEST;
    game_variables->scorpion_stings_found = 0;
    game_variables->save_file_compression_level = 6;
    check_files();
    for(i=0;i<NUM_ITEMS;++i)
	{
	master_items[i][MATRIX_NORMAL] = (struct master_item *)
	    calloc(1, sizeof(struct master_item));
	master_items[i][MATRIX_FINE] = (struct master_item *)
	    calloc(1, sizeof(struct master_item));
	master_items[i][MATRIX_POOR] = (struct master_item *)
	    calloc(1, sizeof(struct master_item));
	*master_items[i][MATRIX_NORMAL] = orig_items[i];
	*master_items[i][MATRIX_FINE] = orig_items[i];
	make_fine(master_items[i][MATRIX_FINE]);
	*master_items[i][MATRIX_POOR] = orig_items[i];
	make_poor(master_items[i][MATRIX_POOR]);
	}
    refresh();
    noecho();
    cbreak();
	/*
    if (LINES < 24 || COLS < 80)
	{
	perror("Error: insufficient terminal size.  The Minstrel's Song needs at least 24 rows\nand 80 columns.");
	my_finish(1);
	}
	*/
    read_file(FILE_MOTD);
    clear();
    scrollok(stdscr, 1);
    top_text=subwin(stdscr, 6, COLS, 0, 0);
    scrollok(top_text, 1);
    left_panel=subwin(stdscr, 18, 11, 6, 0);
    bottom_panel=subwin(stdscr, 3, COLS-14, LINES-3, 13);
    dungeon=subwin(stdscr, LINES-10, COLS-16, 6, 13);
    puzzle=subwin(stdscr, LINES-7, COLS, 6, 0);
    flag_set(&game_variables->flags, PLAYER_AT_CENTER|ASK_PICKUP|
      FREE_UNNEEDED_LEVELS|ALWAYS_DISPLAY_STAIRCASES);
    generate_player();
    pmsg("Welcome to The Minstrel's Song.");
    pmsg("(Please wait one moment, as I prepare things for you...)");
    pmsg("");
    refresh();
    current_level = make_level(LEVEL_COUNTRYSIDE, 0, LOAD, 0);
    /*
    current_level = make_level(LEVEL_DUNGEON, 0, 0, 0);
    */
    no_hostile_add_creature(current_level, player,
      game_variables->new_level_player_x, game_variables->new_level_player_y);
    change_level(player->x, player->y);
    main_city = current_level;
    game_variables->messages_printed = 0;
    pmsg("You arrive at a great Urvanovestilli city, where you have heard \
there is a");
    nocap_pmsg("mentor who will help you on your quests.");
    touchwin(stdscr);
    update_window();
    morewait();
    pmsg("");
    pmsg("Would you like a general hint about the game?");
    if (confirm_silent())
	hint();
    pmsg("");
    pmsg("Hit '?' for help.");
    flag_reset(&game_variables->flags, DURING_INIT);
    /*
    for(i = 0; i < NUM_CREATURES; ++i)
	{
	sprintf(message, "%ld %ld", i, master_creatures[i].race);
	fs_pmsg(message);
	}
    */
    }

void init_restore(char *save_filename)
    {
    long i;
    initscr();
    master_clist = NULL;
    master_ilist = NULL;
    master_llist = NULL;
    game_variables=(struct game_variables_struct *) 
      calloc(1, sizeof(struct game_variables_struct));
#ifdef DEBUG
    game_variables->me = game_variables;
#endif
    game_variables->flags=DURING_INIT;
    game_variables->indent_width = INDENT_DEFAULT;
    game_variables->messages_printed = 0;
    game_variables->fs_messages_printed = 0;
    check_files();
    for(i=0;i<NUM_ITEMS;++i)
	{
	master_items[i][MATRIX_NORMAL] = (struct master_item *)
	  calloc(1, sizeof(struct master_item));
	master_items[i][MATRIX_FINE] = (struct master_item *)
	  calloc(1, sizeof(struct master_item));
	master_items[i][MATRIX_POOR] = (struct master_item *)
	  calloc(1, sizeof(struct master_item));
	*master_items[i][MATRIX_NORMAL] = orig_items[i];
	*master_items[i][MATRIX_FINE] = orig_items[i];
	make_fine(master_items[i][MATRIX_FINE]);
	*master_items[i][MATRIX_POOR] = orig_items[i];
	make_poor(master_items[i][MATRIX_POOR]);
	}
    tsrandom(); /* tsrandom in machdep.c */
    noecho();
    cbreak();
	/*
    if (LINES < 24 || COLS < 80)
	{
	perror("Error: insufficient terminal size.  The Minstrel's Song needs at least 24 rows\nand 80 columns.");
	my_finish(1);
	}
	*/
    read_file(FILE_MOTD);
    refresh();
    scrollok(stdscr, 1);
    top_text=subwin(stdscr, 6, COLS, 0, 0);
    scrollok(top_text, 1);
    left_panel=subwin(stdscr, 18, 11, 6, 0);
    bottom_panel=subwin(stdscr, 3, COLS-14, LINES-3, 13);
    dungeon=subwin(stdscr, LINES-10, COLS-16, 6, 13);
    puzzle=subwin(stdscr, LINES-7, COLS, 6, 0);
    flag_set(&game_variables->flags, PLAYER_AT_CENTER|ASK_PICKUP|
      MESSAGES_PROMPT|FREE_UNNEEDED_LEVELS);
    restore(save_filename);
    flag_reset(&game_variables->flags, DURING_INIT);
    refresh_screen();
    }

void my_finish(long status)
    {
    if (status == 0)
	{
	clear();
	printw("\nBye!\n\n");
	}
    nocbreak();
    echo();
    refresh();
    endwin();
    exit(status);
    }

void generate_player(void)
    {
    char resp;
    long age, age_class, race, gender, i;
    clear();
    printw("Welcome to the world Espiriticthus.  Would you like to:\n");
    printw("\n");
    printw("\ta: have a character generated for you,\n");
    printw("\tb: play yourself,\n");
    printw("\tc: specify attributes, or\n");
    printw("\td: load a saved character?\n");
    printw("\n");
    refresh();
    for(resp = 'e'; ((resp < 'a') || (resp > 'd')); resp = tgetc());
    switch(resp)
	{
	case 'a':
	    printw("What race would you like to be?\n");
	    printw("\ta: Janra\n\tb: Jec\n\tc: Nor'krin\n\td: Shal\n\te: Tuz\n\tf: Urvanovestilli\n\tg: Yedidia\n");
	    refresh();
	    for(resp=tgetc(); (resp < 'a' || resp > 'g'); resp=tgetc());
	    race=resp-'a';
	    printw("\nWhat gender would you like to be?\n\ta: male\n\tb: female\n");
	    refresh();
	    for(resp=tgetc(); (resp < 'a' || resp > 'b'); resp=tgetc());
	    gender=resp-'a';
	    printw("\nAnd what age would you like to be?\n\t(%d-%d)\n",age_class_years[race][CHILD]/2,age_class_years[race][NUM_AGE_CLASSES-1]);
	    refresh();
	    echo();
	    for(scanw("%d",&age); (age < age_class_years[race][CHILD]/2
	      || age > age_class_years[race][NUM_AGE_CLASSES-1]);
		scanf("%ld",&age));
	    noecho();
	    clear();
	    age_class=0;
	    if (age > age_class_years[race][0]) age_class=1;
	    if (age > age_class_years[race][1]) age_class=2;
	    if (age > age_class_years[race][2]) age_class=3;
	    if (age > age_class_years[race][3]) age_class=4;
	    if (age > age_class_years[race][4]) age_class=5;
	    player=extended_creature_create(race, age_class, gender);
	    player->age=age;
	    player->extended->age=age;
	    player->extended->unspent_ep=30;
	    player->extended->image='@';
	    strcpy(player->extended->title, "you");
	    printw("Save character for future play?");
	    refresh();
	    if (confirm_silent_internal())
		save_player();
	break;
	case 'b':
	    interview();
	    player->extended->unspent_ep=30;
	    player->extended->image='@';
	    strcpy(player->extended->title, "you");
	    flag_set(&player->flags, BELLICOSE);
	    strcpy(player->extended->title, "you");
	    printw("Save character for future play?");
	    refresh();
	    if (confirm_silent_internal())
		save_player();
	    read_file(FILE_PLAYER_ENTERS_GAME);
	break;
	case 'c':
	    printw("What race would you like to be?\n");
	    printw("\ta: Janra\n\tb: Jec\n\tc: Nor'krin\n\td: Shal\n\te: Tuz\n\tf: Urvanovestilli\n\tg: Yedidia\n");
	    refresh();
	    for(resp=tgetc(); (resp < 'a' || resp > 'g'); resp=tgetc());
	    race=resp-'a';
	    printw("\nWhat gender would you like to be?\n\ta: male\n\tb: female\n");
	    refresh();
	    for(resp=tgetc(); (resp < 'a' || resp > 'b'); resp=tgetc());
	    gender=resp-'a';
	    printw("\nAnd what age would you like to be?\n\t(%d-%d)\n",age_class_years[race][CHILD]/2,age_class_years[race][NUM_AGE_CLASSES-1]);
	    refresh();
	    echo();
	    for(scanw("%d",&age); (age < age_class_years[race][CHILD]/2
	      || age > age_class_years[race][NUM_AGE_CLASSES-1]);
		scanf("%ld",&age));
	    noecho();
	    clear();
	    age_class=0;
	    if (age > age_class_years[race][0]) age_class=1;
	    if (age > age_class_years[race][1]) age_class=2;
	    if (age > age_class_years[race][2]) age_class=3;
	    if (age > age_class_years[race][3]) age_class=4;
	    if (age > age_class_years[race][4]) age_class=5;
	    player=extended_creature_create(race, age_class, gender);
	    printw("Please enter attributes, unadjusted for race, age, and gender; -25 is the \n");
	    printw("absolute lowest, 0 the average, and 25 the absolute highest.\n");
	    printw("Agility?");
	    refresh();
	    scanw("%ld", &player->extended->perm_attributes[AG]);
	    printw("Ability to Learn?");
	    refresh();
	    scanw("%ld", &player->extended->perm_attributes[AL]);
	    printw("Charisma?");
	    refresh();
	    scanw("%ld", &player->extended->perm_attributes[CH]);
	    printw("Constitution?");
	    refresh();
	    scanw("%ld", &player->extended->perm_attributes[CO]);
	    printw("Dexterity?");
	    refresh();
	    scanw("%ld", &player->extended->perm_attributes[DE]);
	    printw("Intelligence?");
	    refresh();
	    scanw("%ld", &player->extended->perm_attributes[IN]);
	    printw("Knowledge?");
	    refresh();
	    scanw("%ld", &player->extended->perm_attributes[KN]);
	    printw("Memory?");
	    refresh();
	    scanw("%ld", &player->extended->perm_attributes[ME]);
	    printw("Perception?");
	    refresh();
	    scanw("%ld", &player->extended->perm_attributes[PE]);
	    printw("Speed?");
	    refresh();
	    scanw("%ld", &player->extended->perm_attributes[SP]);
	    printw("Strength?");
	    refresh();
	    scanw("%ld", &player->extended->perm_attributes[ST]);
	    printw("Willpower?");
	    refresh();
	    scanw("%ld", &player->extended->perm_attributes[WI]);
	    for(i = 0; i < NUM_ATTRIBUTES; ++i)
		{
		player->extended->perm_attributes[i] = min(
		  player->extended->perm_attributes[i], 25);
		player->extended->perm_attributes[i] = max(
		  player->extended->perm_attributes[i], 0 - 25);
		player->extended->perm_attributes[i] +=
		  racial_attribute_adjustment[race][i];
		player->extended->perm_attributes[i] +=
		  gender_attribute_adjustment[gender][i];
		player->extended->perm_attributes[i] +=
		  age_attribute_adjustment[age_class][i];
		}
	    player->age=age;
	    player->extended->age=age;
	    player->extended->unspent_ep=30;
	    player->extended->image='@';
	    flag_set(&player->flags, BELLICOSE);
	    strcpy(player->extended->title, "you");
	    calc_ready(player);
	    printw("Save character for future play?");
	    refresh();
	    if (confirm_silent_internal())
		save_player();
	break;
	case 'd':
	    load_player();
	break;
	}
    clear();
    }
