/**********************************************************************/
/* The Minstrel's Song is (c) 1998 by Jonathan Hayward.  See the file */
/* lib/copyrigh.txt for a copyright and license statement.            */
/**********************************************************************/

#include <math.h>
#include "tms.h"

/* Populate a level with animals.  */

void populate_animal(struct level *l, long encounter_type, long power_level,
  long frequency)
    {
    long i, j;
    struct small_creature *c;
    power_level = max(power_level, 1);
    for(i=0;i<MAX_X;++i)
	for(j=0;j<MAX_Y;++j)
	    if ((flag_check(l->space[i][j].flags, CAN_OCCUPY)) &&
		(!(flag_check(l->space[i][j].flags, OCCUPIED))) &&
		(trandom()%frequency==0))
		{
		if (encounter_type == DUNGEON_ENCOUNTER)
		    {
		    if (upgrade_check(power_level))
			{
			if (upgrade_check(power_level))
			    {
			    if (upgrade_check(power_level))
				{
				if (upgrade_check(power_level))
				    c = make_random_creature(LEVEL_5_ENCOUNTER);
				else
				    c = make_random_creature(LEVEL_4_ENCOUNTER);
				}
			    else
				c = make_random_creature(LEVEL_3_ENCOUNTER);
			    }
			else
			    c = make_random_creature(LEVEL_2_ENCOUNTER);
			}
		    else
			c = make_random_creature(LEVEL_1_ENCOUNTER);
		    }
		else
		    c=make_random_creature(encounter_type);
		flag_set(&c->flags, INACTIVE);
		c->x=i;
		c->y=j;
		append_creature_to_clist(c,
		  &l->space[i][j].creatures);
		append_creature_to_clist(c, &l->creatures);
		flag_set(&l->space[i][j].flags, OCCUPIED);
		check_addition(l, c->identity, c->x, c->y);
		}
    }

/* Populate a level with items */
void populate_item(struct level *this_level, long item_type, long power_level,
  long frequency)
    {
    long i, j;
    struct small_item *new_item;
    struct ilist *il;
    power_level = max(power_level, 1);
    for(i=0;i<MAX_X;++i)
	for(j=0;j<MAX_Y;++j)
	    if ((flag_check(this_level->space[i][j].flags, CAN_OCCUPY)) &&
		(trandom()%frequency==0))
		{
		new_item = generate_item(this_level, item_type);
		if (!(new_item == NULL))
		    {
		    il=(struct ilist *) calloc(1, sizeof(struct
			ilist));
#ifdef DEBUG
		    il->me = il;
#endif
		    il->current=new_item;
		    il->next=this_level->space[i][j].items;
		    this_level->space[i][j].items = il;
		    }
		}
    }

/* Populate a level with money.  The amount and value of coin increase as we */
/* go deeper and deeper in.  */
void populate_money(struct level *this_level, long power_level, long frequency)
    {
    long i, j, chance_no_increase, chance_total;
    power_level = max(power_level, 1);
    chance_no_increase = chance_total = 32767L;
    for(i=0;i<power_level;++i)
	chance_no_increase = (chance_no_increase * (CHANCE_NEXT_LEVEL - 1)) /
	  (CHANCE_NEXT_LEVEL);
    for(i=0;i<MAX_X;++i)
	for(j=0;j<MAX_Y;++j)
	    if ((flag_check(this_level->space[i][j].flags, CAN_OCCUPY)) &&
		(trandom()%frequency==0))
		{
		if (trandom()%chance_total < chance_no_increase)
		    this_level->space[i][j].cu = random_adjust(power_level
		      * power_level);
		else if (trandom()%chance_total < chance_no_increase)
		    this_level->space[i][j].si = random_adjust(power_level
		      * power_level);
		else if (trandom()%chance_total < chance_no_increase)
		    this_level->space[i][j].el = random_adjust(power_level
		      * power_level);
		else
		    this_level->space[i][j].au = random_adjust(power_level
		      * power_level);
		}
    }

/* set the marked flag for all of the spaces which are land */
/* This is a bit inefficient, but still */
/* O(n); it only recurses if the space is not marked.  (The marking is very */
/* important; without it, we would have an infinite loop.)  Not currently */
/* used.  */

void mark_connected_component(struct level *l, long x, long y)
    {
    long i, j;
    if ((x >= 0) && (x < MAX_X) && (y >= 0) && (y < MAX_Y))
	flag_set(&l->space[x][y].flags, MARKED);
    if ((x > 0) && (x < MAX_X - 1) && (y > 0) && (y < MAX_Y - 1))
	for(i=x-1; i < x + 2; ++i)
	    for(j = y - 1; j < y + 2; ++j)
		if (((!flag_check(l->space[i][j].flags, IS_WATER))) &&
		  (!(flag_check(l->space[i][j].flags, MARKED))))
		    mark_connected_component(l, i, j);
    }

void mark_connected_component_can_occupy(struct level *l, long x, long y)
    {
    long i, j;
    if ((x >= 0) && (x < MAX_X) && (y >= 0) && (y < MAX_Y))
	flag_set(&l->space[x][y].flags, MARKED);
    if ((x > 0) && (x < MAX_X - 1) && (y > 0) && (y < MAX_Y - 1))
	for(i=x-1; i < x + 2; ++i)
	    for(j = y - 1; j < y + 2; ++j)
		if ((!((l->space[i][j].type == L_WALL) && (!(flag_check(
		  l->space[i][j].flags, HAS_SECRET))))) && (!(flag_check(l->
		  space[i][j].flags, MARKED))))
		    mark_connected_component(l, i, j);
    }

void unmark_level(struct level *l)
    {
    long i, j;
    for(i=0;i<MAX_X;++i)
	for(j=0;j<MAX_Y;++j)
	    flag_reset(&l->space[i][j].flags, MARKED);
    }


void mark_circle(struct level *this_level, long x, long y, long radius)
    {
    long i, j;
    if ((x - radius < 0) || (x + radius >= MAX_X) || (y - radius < 0) ||
      (y + radius >= MAX_Y))
	perror("Error: circle does not fit on level.");
    for(i=x - radius; i <= x + radius; ++i)
	for(j=y - radius; j <= y + radius; ++j)
	    if ((((i - x) * (i - x)) + ((j - y) * (j - y))) <= (radius *
	      radius))
		flag_set(&this_level->space[i][j].flags, MARKED);
    }

/* find the coordinates of a point on the boundary of a marked area, via */
/* staggering drunk */

void find_boundary(struct level *l, long *x, long *y)
    {
    long i, j;
    i = *x;
    j = *y;
    while ((flag_check(l->space[i][j].flags, MARKED)) && (i >= 0) && (i <
      MAX_X) && (j >= 0) && (j < MAX_Y))
	{
	*x = i;
	*y = j;
	switch(trandom()%4)
	    {
	    case 0: ++i; break;
	    case 1: --i; break;
	    case 2: ++j; break;
	    case 3: --j; break;
	    }
	}
    }

/* The number of steps is the approximate number of steps taken by the */
/* staggering drunk algorithm, which here sweeps out a circle.  */

void mark_pool(struct level *l, long x, long y, long radius, long steps)
    {
    long i, j;
    i=x;
    j=y;
    while ((i - radius > 0) && (i + radius < MAX_X - 1) && (j - radius > 0) &&
      (j + radius < MAX_Y - 1) && (trandom()%steps > 0))
	{
	mark_circle(l, x, y, radius);
	switch(trandom()%4)
	    {
	    case 0: ++i; break;
	    case 1: --i; break;
	    case 2: ++j; break;
	    case 3: --j; break;
	    }
	x=i;
	y=j;
	}
    }

struct special_struct *new_special(void)
    {
    struct special_struct *special;
    special = (struct special_struct *) calloc(1, sizeof(struct
      special_struct));
#ifdef DEBUG
    special->me = special;
#endif
    special->flags = 0;
    special->level_flags = 0;
    special->type = 0;
    special->x = 0;
    special->y = 0;
    special->shop = NULL;
    special->establishment = NULL;
    special->level_type = NONE;
    special->animal_encounter_type = NONE;
    special->animal_encounter_frequency = NEVER;
    special->human_encounter_type = NONE;
    special->human_encounter_frequency = NEVER;
    special->other = 0;
    special->takes_player_to = NULL;
    special->text = NULL;
    return special;
    }

void change_level(long x, long y)
    {
    char message[256];
    struct level *old_level, *new_level;
    struct clist *clcurr;
    long i, j, point_back, new_power_level;
    old_level = current_level;
    if ((current_level->space[x][y].special == NULL) ||
	(!((current_level->space[x][y].property  == P_ENTRANCE)
	  ||
	(current_level->space[x][y].property  == P_EXIT) ||
	(current_level->space[x][y].property  ==
	  P_PUSHES_LEVEL) ||
	(current_level->space[x][y].property  ==
	  P_STAIRWELL_DOWN) ||
	(current_level->space[x][y].property  ==
	  P_STAIRWELL_UP) ||
	(current_level == countryside) ||
	(flag_check(game_variables->quests_completed, SLEW_TERRASK)))))
	{
	/*
	sprintf(message, "%ld %ld %ld",
	  current_level->space[player->x][player->y].special,
	  current_level->space[player->x][player->y].property, P_PEAK);
	pmsg(message);
	*/
	pmsg("This space is not a stairwell, entrance, or exit.");
	new_level = old_level;
	}
    else
	{
	if ((!(old_level->exit_message==ST_EMPTY)) &&
	  (!(flag_check(game_variables->flags, DURING_INIT))))
	    {
	    find_string(old_level->exit_message, message);
	    pmsg(message);
	    }
	remove_creature(old_level, player);
	/* We remove all of the following creatures from the old level, to */
	/* appear one by one wherever the player appears.  */
	if (!(player->followed_by==NULL))
	    for(clcurr=player->followed_by; (!(clcurr==NULL));
	      clcurr=clcurr->next)
		remove_creature(old_level, clcurr->current);
	if (old_level->space[x][y].special->takes_player_to !=
	  NULL)
	    {
	    new_level=
	      old_level->space[x][y].special->takes_player_to;
	    game_variables->new_level_player_x = 
	      old_level->space[x][y].special->x;
	    game_variables->new_level_player_y = 
	      old_level->space[x][y].special->y;
	    }
	else
	    {
	    flag_reset(&game_variables->flags, GOING_UP);
	    switch(old_level->space[x][y].type)
		{
		case L_STAIRWELL_UP:
		    new_power_level = old_level->power_level - 1;
		    flag_set(&game_variables->flags, GOING_UP);
		break;
		case L_STAIRWELL_DOWN:
		    new_power_level = old_level->power_level + 1;
		break;
		default:
		    new_power_level = old_level->power_level;
		break;
		}
	    new_level = make_level(
		old_level->space[x][y].special->level_type,
		new_power_level,
		old_level->space[x][y].special->level_flags,
		old_level->space[x][y].special->other);
	    new_level->animal_encounter_type = old_level->space[x][y].special->
	      animal_encounter_type;
	    new_level->animal_encounter_frequency = old_level->space[x][y].
	      special->animal_encounter_frequency;
	    new_level->human_encounter_type = old_level->space[x][y].special->
	      human_encounter_type;
	    new_level->human_encounter_frequency = old_level->space[x][y].
	      special->human_encounter_frequency;
	    if (old_level->race != NONE)
		new_level->race = old_level->race;
	    if ((old_level->space[x][y].special != NULL) &&
	      (old_level->space[x][y].property == P_ENTRANCE))
		if (old_level->space[x][y].special->establishment != NULL)
		    new_level->establishment =
		      old_level->space[x][y].special->establishment;
	    for(i=0;i<MAX_X;++i)
		for(j=0;j<MAX_Y;++j)
		    {
		    point_back = 0;
		    switch(old_level->space[x][y].type)
			{
			case L_STAIRWELL_DOWN:
			    if (new_level->space[i][j].property ==
			      P_STAIRWELL_UP)
				point_back = 1;
			break;
			case L_STAIRWELL_UP:
			    if (new_level->space[i][j].property ==
			      P_STAIRWELL_DOWN)
				point_back = 1;
			break;
			default:
			    if (new_level->space[i][j].property == P_EXIT)
				point_back = 1;
			break;
			}
		    if ((point_back == 1) && (new_level->space[i][j].special
		      != 0))
			{
			new_level->space[i][j].special->takes_player_to =
			  old_level;
			new_level->space[i][j].special->x = x;
			new_level->space[i][j].special->y = y;
			}
		    }
	    if (!(flag_check(new_level->flags, ALWAYS_FREE)))
		{
		old_level->space[x][y].special->x = 
		  game_variables->new_level_player_x;
		old_level->space[x][y].special->y = 
		  game_variables->new_level_player_y;
		old_level->space[x][y].special->takes_player_to =
		  new_level;
		}
	    }
	/* We may want to free the old level, because levels take a lot of */
	/* memory.  */
	if ((flag_check(game_variables->flags, FREE_UNNEEDED_LEVELS)) &&
	  (((!(flag_check(old_level->flags, NO_FREE))) &&
	  (((old_level->space[x][y].property == P_EXIT) ||
	  (abs(old_level->power_level) > abs(new_level->power_level))))) ||
	  (flag_check(old_level->flags, ALWAYS_FREE))))
	    {
	    for(i=0;i<MAX_X;++i)
		for(j=0;j<MAX_Y;++j)
		    if (!(new_level->space[i][j].special==NULL))
			if (new_level->space[i][j].special->takes_player_to ==
			  old_level)
			    new_level->space[i][j].special->takes_player_to =
			      NULL;
	    free_level(old_level);
	    }
	add_creature(new_level, player, game_variables->new_level_player_x,
	  game_variables->new_level_player_y);
	current_level = new_level;
	update_light_sources();
	if (find_light_source_radius(player) > 0)
	    add_new_light_source_to_creature(player,
	      find_light_source_radius(player));
	if ((!(new_level->entrance_message==ST_EMPTY)) &&
	  (!(flag_check(game_variables->flags, DURING_INIT))))
	    {
	    find_string(new_level->entrance_message, message);
	    pmsg(message);
	    }
	check_level_pointers(new_level);
	}
    }

void add_shop(struct level *this_level, long x, long y, long shop_type)
    {
    struct shop_struct *result;
    result = (struct shop_struct *) calloc(1, sizeof(struct shop_struct));
#ifdef DEBUG
    result->me = result;
#endif
    result->merchandise = NONE;
    result->flags = NOT_STOCKED;
    result->markup = 0;
    result->buyback = 0;
    result->items = NULL;
    result->creatures = NULL;
    switch(this_level->race)
	{
	case JEC:
	    result->markup = 20;
	    result->buyback = 50;
	break;
	case SHAL:
	    result->markup = 20;
	    result->buyback = 40;
	break;
	case TUZ:
	    result->markup = 50;
	    result->buyback = 60;
	break;
	case URVANOVESTILLI:
	    result->markup = 150;
	    result->buyback = 20;
	break;
	case YEDIDIA:
	    result->markup = 75;
	    result->buyback = 30;
	break;
	}
    switch(shop_type)
	{
	case ADVENTURERS_STORE:
	    result->merchandise = DUNGEON_ITEM_NORMAL;
	    this_level->space[x][y].sign = ST_EQUIPMENT_FOR_ADVENTURERS;
	    flag_set(&result->flags, BUY_ALL);
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case ADVENTURERS_STORE_FINE:
	    result->merchandise = DUNGEON_ITEM_FINE;
	    result->markup *= 2;
	    this_level->space[x][y].sign =
	      ST_EQUIPMENT_OF_CALIPER_FOR_SERIOUS_ADVENTURERS;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case ARMORER:
	    result->merchandise = ARMOR_NORMAL;
	    this_level->space[x][y].sign = ST_ARMOR_FOR_EVERYONE;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case ARMORER_FINE:
	    result->merchandise = ARMOR_FINE;
	    result->markup *= 2;
	    this_level->space[x][y].sign =
	      ST_ARMOR_OF_CALIPER_FOR_SERIOUS_ADVENTURERS;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case BOAT_SHOP:
	    result->merchandise = BOAT_NORMAL;
	    this_level->space[x][y].sign = ST_THE_SAILORS_CRAG;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case BOOKSTORE:
	    result->merchandise = BOOKSTORE_ITEM_NORMAL;
	    this_level->space[x][y].sign = ST_THE_BOOK_LOVERS_CORNER;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case BOOKSTORE_FINE:
	    result->merchandise = BOOKSTORE_ITEM_FINE;
	    result->markup *= 2;
	    this_level->space[x][y].sign =
	      ST_BOOKS_OF_CALIPER_FOR_SERIOUS_STUDENTS;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case BRIC_A_BRAC:
	    result->merchandise = NON_EQUIPMENT_TREASURE_NORMAL;
	    this_level->space[x][y].sign =
	      ST_A_LITTLE_OF_THIS_A_LITTLE_OF_THAT;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case DEVICE_SHOP:
	    result->merchandise = DEVICE_NORMAL;
	    this_level->space[x][y].sign =
	      ST_YOUR_SOURCE_FOR_CLOCKWORK_DEVICES;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case DEVICE_SHOP_FINE:
	    result->merchandise = DEVICE_FINE;
	    result->markup *= 2;
	    this_level->space[x][y].sign = ST_CLOCKWORK_DEVICES_OF_CALIPER;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case FURNITURE_STORE:
	    result->merchandise = FURNITURE_NORMAL;
	    this_level->space[x][y].sign = ST_WE_HELP_MAKE_A_HOUSE_A_HOME;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case HERBALIST_SHOP:
	    result->merchandise = HERBALIST_ITEM_NORMAL;
	    this_level->space[x][y].sign =
	      ST_SHARING_NATURES_SECRETS_FOR_GENERATIONS;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case HERBALIST_SHOP_FINE:
	    result->merchandise = HERBALIST_ITEM_FINE;
	    result->markup *= 2;
	    this_level->space[x][y].sign = ST_THE_BEST_IN_RARE_HERBS;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case HORSE_SHOP:
	    result->merchandise = HORSE;
	    this_level->space[x][y].sign =
	      ST_PROVIDING_FINE_HORSES_TO_EVERY_CUSTOMER;
	    flag_set(&result->flags, CARRIES_ANIMALS);
	break;
	case JEWELLER:
	    result->merchandise = JEWELLER_ITEM_NORMAL;
	    this_level->space[x][y].sign = ST_YE_OLDE_JEWELLERS_SHOPPE;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case LOCATOR:
	    result->merchandise = BUILDING_ITEM_NORMAL;
	    result->markup *= 3;
	    this_level->space[x][y].sign =
	      ST_WE_LOCATE_RARE_AND_HARD_TO_FIND_ITEMS;
	    flag_set(&result->flags, CARRIES_ITEMS);
	    flag_set(&result->flags, CAN_ACQUIRE_ANYTHING);
	    /* This shop doesn't have anything in stock unless ordered.  */
	    flag_reset(&result->flags, NOT_STOCKED);
	break;
	case MISCELLANEOUS_EQUIPMENT:
	    result->merchandise = NON_DEVICE_EQUIPMENT_NORMAL;
	    this_level->space[x][y].sign =
	      ST_MISCELLANEOUS_EQUIPMENT_FOR_ADVENTURERS;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case PET_SHOP:
	    result->merchandise = SHAL_ENCOUNTER;
	    this_level->space[x][y].sign = ST_FRIENDLY_PETS_SOLD_HERE;
	    flag_set(&result->flags, CARRIES_ANIMALS);
	break;
	case RANDOM_CREATURE_SHOP:
	    result->merchandise = TUZ_ENCOUNTER;
	    this_level->space[x][y].sign = ST_THE_LION_TAMERS_DEN;
	    flag_set(&result->flags, CARRIES_ANIMALS);
	    flag_set(&result->flags, BUY_ALL);
	break;
	case SUPPLY_STORE:
	    result->merchandise = SUPPLIES_NORMAL;
	    this_level->space[x][y].sign =
	      ST_SUPPLIES_TO_MEET_YOUR_DAY_TO_DAY_NEEDS;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case WEAPONSMITH:
	    result->merchandise = WEAPON_NORMAL;
	    this_level->space[x][y].sign = ST_WEAPONS_FOR_THE_COMMON_MAN;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	case WEAPONSMITH_FINE:
	    result->merchandise = WEAPON_FINE;
	    this_level->space[x][y].sign =
	      ST_WEAPONS_OF_CALIPER_FOR_SERIOUS_ADVENTURERS;
	    flag_set(&result->flags, CARRIES_ITEMS);
	break;
	}
    /* We don't stock the shop yet, because stocking all the shops at once is */
    /* time consuming, +takes memory.  */
    if (this_level->space[x][y].special == NULL)
	this_level->space[x][y].special = new_special();
    this_level->space[x][y].special->shop = result;
    }

/* Stock a shop.  */

void stock_shop(struct shop_struct *s)
    {
    long my_continue, average_items, i;
    struct small_item *new_item;
    struct ilist *new_ilist;
    struct small_creature *new_creature;
    struct clist *new_clist;
    switch(current_level->race)
	{
	case JEC:
	    average_items = 10;
	break;
	case SHAL:
	    average_items = 5;
	break;
	case TUZ:
	    average_items = 20;
	break;
	case URVANOVESTILLI:
	    average_items = 50;
	break;
	case YEDIDIA:
	    average_items = 15;
	break;
	default:
	    average_items = 10;
	break;
	}
    flag_reset(&s->flags, NOT_STOCKED);
    if (flag_check(s->flags, CARRIES_ITEMS))
	{
	my_continue = 1;
	i = 0;
	while (my_continue == 1)
	    {
	    new_item = generate_item(current_level, s->merchandise);
	    if (flag_check(new_item->flags, I_UNIQUE))
		while (flag_check(new_item->flags, I_UNIQUE))
		    {
		    free_item(new_item);
		    new_item = generate_item(current_level, s->merchandise);
		    }
	    if (new_item->contents != NULL)
		{
		free_items(new_item->contents);
		new_item->contents = NULL;
		}
	    new_item->price = random_adjust((((new_item->value * (100 +
	      s->markup))/100)*(100 + s->markup))/100);
	    new_ilist = (struct ilist *) calloc(1, sizeof(struct ilist));
#ifdef DEBUG
	    new_ilist->me = new_ilist;
#endif
	    new_ilist->current = new_item;
	    new_ilist->next = s->items;
	    s->items = new_ilist;
#ifdef DEBUG
	    check_pointer((struct pointer_struct *) new_item);
	    check_pointer((struct pointer_struct *) new_item->tname);
	    check_pointer((struct pointer_struct *) new_ilist);
#endif
	    flag_set(&new_item->flags, IDENTIFIED);
	    if (trandom() % average_items == 0)
		my_continue = 0;
	    ++i;
	    if (i > 62)
		my_continue = 0;
	    }
	}
    if (flag_check(s->flags, CARRIES_ANIMALS))
	{
	my_continue = 1;
	while (my_continue == 1)
	    {
	    new_creature = make_random_creature(s->merchandise);
	    if (flag_check(new_creature->flags, UNIQUE))
		while (flag_check(new_creature->flags, UNIQUE))
		    {
		    free_creature(new_creature);
		    new_creature = make_random_creature(s->merchandise);
		    }
	    new_creature->price = random_adjust((((
	      master_creatures[new_creature->identity].value * (100 +
	      s->markup))/100)*(100 + s->markup))/100);
	    new_clist = (struct clist *) calloc(1, sizeof(struct clist));
#ifdef DEBUG
	    new_clist->me = new_clist;
#endif
	    new_clist->current = new_creature;
	    new_clist->next = s->creatures;
	    s->creatures = new_clist;
	    if (trandom() % average_items == 0)
		my_continue = 0;
	    }
	}
    }

void assign_door(struct level *this_level, long x, long y, long door_id)
    {
    struct space_struct *space;
    long flags;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) this_level);
#endif
    flags = 0;
    space = &this_level->space[x][y];
    if (!(space->special==NULL))
	free_special(space->special);
    space->special = NULL;
    flag_set(&space->flags, OTHER_IS_LOCATION_TYPE);
    space->other = door_id;
    flag_set(&space->flags, AUTOMATIC_EFFECT);
    if (door_id <= MAX_SHOP)
	{
	space->special = new_special();
	add_shop(this_level, x, y, door_id);
	space->property = P_SHOP;
	}
    else
	switch(door_id)
	    {
	    case ADVENTURERS_GUILD:
		space->property = P_ADVENTURERS_GUILD;
		space->sign = ST_THE_ADVENTURERS_GUILD;
	    break;
	    case CATHEDRAL:
		change_space(space, S_CATHEDRAL_ENTRANCE);
	    break;
	    case CLOCKWORK_DOOR:
		change_space(space, S_CLOCKWORK_DOOR);
	    break;
	    case CLOSED_DOOR:
		change_space(space, S_CLOSED_DOOR);
	    break;
	    case DOOR_OPEN_DOOR:
		change_space(space, S_OPEN_DOOR);
	    break;
	    case HOUSE:
		change_space(space, S_HOUSE_DOOR);
	    break;
	    case LOCKED_DOOR:
		change_space(space, S_LOCKED_DOOR);
	    break;
	    case MAPMAKER:
		space->property = P_MAPMAKER;
		space->sign = ST_THE_MAPMAKERS_SHOP;
	    break;
	    case MENTOR:
		space->property = P_MENTOR;
	    break;
	    case MEDICS:
		space->property = P_MEDIC;
		space->sign = ST_MEDICAL_HOSPICE_FOR_THE_SICK_AND_WOUNDED;
	    break;
	    case RESTAURANT:
		flag_set(&space->flags, AUTOMATIC_EFFECT);
		space->property = P_RESTAURANT;
		space->special = new_special();
		switch(trandom() % 4)
		    {
		    case 0:
			space->sign = ST_CIRCLE_OF_FRIENDS;
		    break;
		    case 1:
			space->sign = ST_THE_KINGS_COURT;
		    break;
		    case 2:
			space->sign = ST_THE_MOONLIT_PATH;
		    break;
		    case 3:
			space->sign = ST_THE_SILVER_RING;
		    break;
		    }
		switch(this_level->race)
		    {
		    case JEC: flags = ESTABLISHMENT_JEC_RESTAURANT; break;
		    case SHAL: flags = ESTABLISHMENT_SHAL_RESTAURANT; break;
		    case TUZ: flags = ESTABLISHMENT_TUZ_RESTAURANT; break;
		    case URVANOVESTILLI:
			flags = ESTABLISHMENT_URVANOVESTILLI_RESTAURANT;
		    break;
		    case YEDIDIA:
			flags = ESTABLISHMENT_YEDIDIA_RESTAURANT;
		    break;
		    }
		space->special->establishment = new_establishment(flags,
		  this_level->race);
	    break;
	    case SECRET_PASSAGE:
		change_space(space, S_SECRET_PASSAGE);
	    break;
	    case TAVERN:
		flag_set(&space->flags, AUTOMATIC_EFFECT);
		space->property = P_ENTRANCE;
		space->special = new_special();
		switch(trandom() % 6)
		    {
		    case 0:
			space->sign = ST_RYANS_CORNER;
		    break;
		    case 1:
			space->sign = ST_THE_BOARS_HEAD;
		    break;
		    case 2:
			space->sign = ST_THE_DRAGONS_DEN;
		    break;
		    case 3:
			space->sign = ST_THE_GRIFFIN;
		    break;
		    case 4:
			space->sign = ST_THE_LIONHEART;
		    break;
		    case 5:
			space->sign = ST_THE_SILVERFISH;
		    break;
		    }
		space->special->level_type = LEVEL_TAVERN;
		flag_set(&space->special->level_flags, LOAD);
		switch(this_level->race)
		    {
		    case JEC: flags = ESTABLISHMENT_JEC_RESTAURANT|
		      ESTABLISHMENT_JEC_TAVERN; break;
		    case SHAL: flags = ESTABLISHMENT_SHAL_RESTAURANT|
		      ESTABLISHMENT_SHAL_TAVERN; break;
		    case TUZ: flags = ESTABLISHMENT_TUZ_RESTAURANT|
		      ESTABLISHMENT_TUZ_TAVERN; break;
		    case URVANOVESTILLI:
			flags = ESTABLISHMENT_URVANOVESTILLI_RESTAURANT|
			ESTABLISHMENT_URVANOVESTILLI_TAVERN;
		    break;
		    case YEDIDIA:
			flags = ESTABLISHMENT_YEDIDIA_RESTAURANT|
			  ESTABLISHMENT_YEDIDIA_TAVERN;
		    break;
		    }
		space->special->establishment = new_establishment(flags,
		  this_level->race);
	    break;
	    case TINKERERS_GUILD:
		space->property = P_TINKERERS_GUILD;
		space->sign = ST_THE_TINKERERS_GUILD;
	    break;
	    }
    }

struct establishment_struct *new_establishment(long flags, long race)
    {
    long i;
    struct establishment_struct *result;
    result = (struct establishment_struct *) calloc(1, sizeof(struct
      establishment_struct));
#ifdef DEBUG
    result->me = result;
#endif
    for(i = 0; i < NUM_ESTABLISHMENT_ITEMS; ++i)
	{
	result->flags[i] = 0;
	if (flag_check(establishment_info[i].flags, flags))
	    if (trandom() % 100 < establishment_info[i].chance)
		flag_set(&result->flags[i], ESTABLISHMENT_ITEM_AVAILABLE);
	switch(race)
	    {
	    case JEC: case SHAL:
		result->price[i] = random_adjust(establishment_info[i].price /
		  2);
	    break;
	    case URVANOVESTILLI:
		result->price[i] = random_adjust(establishment_info[i].price *
		  2);
	    break;
	    default:
		result->price[i] = random_adjust(establishment_info[i].price);
	    break;
	    }
	}
    return result;
    }

void prepare_room_space(struct level *l, struct space_struct *s, long room_id)
    {
    long higher_power_level, i;
    struct small_item *this_item;
    struct ilist *this_ilist;
    higher_power_level = 0;
    for(i = 0; i < l->power_level; ++i)
	if (trandom() % CHANCE_NEXT_LEVEL == 0)
	    ++higher_power_level;
    this_item = NULL;
    switch(room_id)
	{
	case ROOM_ARMORY:
	    s->room_description = ST_AN_ARMORY;
	    if (trandom() % 2 == 0)
		switch(trandom() % 3)
		    {
		    case 0: case 1:
			if (higher_power_level)
			    this_item = generate_item(l, ARMOR_FINE);
			else
			    this_item = generate_item(l, ARMOR_NORMAL);
		    break;
		    case 2:
			if (higher_power_level)
			    this_item = generate_item(l, WEAPON_FINE);
			else
			    this_item = generate_item(l, WEAPON_NORMAL);
		    break;
		    }
	break;
	case ROOM_ART_GALLERY:
	    s->room_description = ST_AN_ART_GALLERY;
	    if (trandom() % 10 == 0)
		{
		if (higher_power_level)
		    this_item = generate_item(l, ART_FINE);
		else
		    this_item = generate_item(l, ART_NORMAL);
		}
	break;
	case ROOM_ART_STUDIO:
	    s->room_description = ST_AN_ART_STUDIO;
	    if (trandom() % 6 == 0)
		{
		if (higher_power_level)
		    this_item = generate_item(l, ART_FINE);
		else
		    this_item = generate_item(l, ART_NORMAL);
		}
	break;
	case ROOM_BARE_ROOM:
	    s->room_description = ST_A_BARE_ROOM;
	break;
	case ROOM_BEDROOM:
	    if (trandom() % 10 == 0)
		{
		change_space(s, S_BED);
		}
	    s->room_description = ST_A_BEDROOM;
	    if (trandom() % 20 == 0)
		{
		if (higher_power_level)
		    this_item = generate_item(l, BUILDING_ITEM_FINE);
		else
		    this_item = generate_item(l, BUILDING_ITEM_NORMAL);
		}
	break;
	case ROOM_CHEMICAL_LABORATORY:
	    if (trandom() % 5 == 0)
		change_space(s, S_TABLE);
	    else if (trandom() % 10 == 0)
		change_space(s, S_CHAIR);
	    s->room_description = ST_A_CHEMICAL_LABORATORY;
	    flag_set(&s->flags, CAN_CHEM);
	break;
	case ROOM_CLOAKROOM:
	    s->room_description = ST_A_CLOAKROOM;
	    if (trandom() % 2 == 0)
		this_item = generate_item(l, CLOTHING);
	break;
	case ROOM_CLOSET:
	    s->room_description = ST_A_CLOSET;
	    if (trandom() % 2 == 0)
		this_item = generate_item(l, CLOTHING);
	    else if (trandom() % 4 == 0)
		{
		if (higher_power_level)
		    this_item = generate_item(l, BUILDING_ITEM_FINE);
		else
		    this_item = generate_item(l, BUILDING_ITEM_NORMAL);
		}
	break;
	case ROOM_DEN:
	    if (trandom() % 4 == 0)
		change_space(s, S_CHAIR);
	    else if (trandom() % 8 == 0)
		change_space(s, S_TABLE);
	    else if (trandom() % 16 == 0)
		change_space(s, S_FOUNTAIN);
	    s->room_description = ST_A_DEN;
	break;
	case ROOM_DEVICE_LABORATORY:
	    if (trandom() % 4 == 0)
		change_space(s, S_CABINET);
	    else if (trandom() % 4 == 0)
		change_space(s, S_TABLE);
	    else if (trandom() % 4 == 0)
		change_space(s, S_CHAIR);
	    s->room_description = ST_A_DEVICE_LABORATORY;
	    if (trandom() % 10 == 0)
		{
		if (higher_power_level)
		    this_item = generate_item(l, DEVICE_FINE);
		else
		    this_item = generate_item(l, DEVICE_NORMAL);
		}
	    flag_set(&s->flags, CAN_TINKER);
	break;
	case ROOM_GREAT_HALL:
	    if (trandom() % 4 == 0)
		change_space(s, S_CHAIR);
	    s->room_description = ST_A_GREAT_HALL;
	break;
	case ROOM_HALLWAY:
	    s->room_description = ST_A_HALLWAY;
	break;
	case ROOM_KITCHEN:
	    if (trandom() % 4 == 0)
		change_space(s, S_TABLE);
	    s->room_description = ST_A_KITCHEN;
	break;
	case ROOM_LIBRARY:
	    s->room_description = ST_A_LIBRARY;
	    if (trandom() % 2 == 0)
		{
		if (higher_power_level)
		    this_item = generate_item(l, BOOKSTORE_ITEM_FINE);
		else
		    this_item = generate_item(l, BOOKSTORE_ITEM_NORMAL);
		}
	break;
	case ROOM_MECHANICAL_ROOM:
	    change_space(s, S_MECHANICAL);
	    s->room_description = ST_A_MECHANICAL_ROOM;
	break;
	case ROOM_MUSIC_ROOM:
	    s->room_description = ST_A_MUSIC_ROOM;
	    if (trandom() % 8 == 0)
		{
		if (higher_power_level)
		    this_item = generate_item(l, MUSICAL_INSTRUMENT_FINE);
		else
		    this_item = generate_item(l, MUSICAL_INSTRUMENT_NORMAL);
		}
	break;
	case ROOM_PARLOR:
	    if (trandom() % 8 == 0)
		change_space(s, S_CHAIR);
	    else if (trandom() % 20 == 0)
		change_space(s, S_TABLE);
	    s->room_description = ST_A_PARLOR;
	break;
	case ROOM_ROOM_WITH_A_DIAS:
	    if (trandom() % 5 == 0)
		{
		change_space(s, S_CHAIR);
		s->space_description = ST_A_DIAS;
		}
	    s->room_description = ST_A_ROOM_WITH_A_DIAS;
	break;
	case ROOM_ROOM_WITH_MURALS:
	    s->room_description = ST_A_ROOM_WITH_MURALS;
	break;
	case ROOM_STOREROOM:
	    s->room_description = ST_A_STOREROOM;
	    if (trandom() % 2 == 0)
		{
		if (trandom() % 4)
		    {
		    if (higher_power_level)
			this_item = generate_item(l, SUPPLIES_FINE);
		    else
			this_item = generate_item(l, SUPPLIES_NORMAL);
		    }
		else
		    {
		    if (higher_power_level)
			this_item = generate_item(l, BUILDING_ITEM_FINE);
		    else
			this_item = generate_item(l, BUILDING_ITEM_NORMAL);
		    }
		}
	break;
	case ROOM_TREASURY:
	    s->room_description = ST_A_TREASURY;
	    if (trandom() % 3)
		{
		if (trandom() % 3)
		    {
		    this_item = generate_item(l, GEM);
		    if (higher_power_level)
			{
			small_item_make_normal(this_item);
			small_item_make_fine(this_item);
			}
		    }
		else
		    {
		    this_item = generate_item(l, JEWELRY);
		    if (higher_power_level)
			{
			small_item_make_normal(this_item);
			small_item_make_fine(this_item);
			}
		    }
		}
	    switch(abs(l->power_level) + higher_power_level)
		{
		case 0:
		break;
		case 1:
		    s->cu = 1 + trandom() % 10 + trandom() % 10;
		break;
		case 2:
		    s->cu = 1 + trandom() % 10 + trandom() % 10;
		    s->si = 1 + trandom() % 10 + trandom() % 10;
		break;
		case 3:
		    s->cu = 1 + trandom() % 10 + trandom() % 10;
		    s->si = 1 + trandom() % 10 + trandom() % 10;
		    s->el = 1 + trandom() % 10 + trandom() % 10;
		break;
		case 4:
		    s->cu = 1 + trandom() % 10 + trandom() % 10;
		    s->si = 1 + trandom() % 10 + trandom() % 10;
		    s->el = 1 + trandom() % 10 + trandom() % 10;
		    s->au = 1 + trandom() % 10 + trandom() % 10;
		break;
		default:
		    s->cu = 1 + trandom() % 10 + trandom() % 10;
		    s->si = 1 + trandom() % 10 + trandom() % 10;
		    s->el = 1 + trandom() % 10 + trandom() % 10;
		    s->au = 1 + trandom() % (long) pow(10, abs(l->power_level)
		      + higher_power_level - 3) + trandom() % (long) pow(10,
		      abs(l->power_level) + (higher_power_level) - 3);
		break;
		}
	break;
	}
    if (this_item != NULL)
	{
	this_ilist = (struct ilist *) calloc(1, sizeof(struct ilist));
#ifdef DEBUG
	this_ilist->me = this_ilist;
#endif
	this_ilist->next = NULL;
	this_ilist->current = this_item;
	append_ilist(&s->items, this_ilist);
	}
    }
