/**********************************************************************/
/* The Minstrel's Song is (c) 1998 by Jonathan Hayward.  See the file */
/* lib/copyrigh.txt for a copyright and license statement.            */
/**********************************************************************/

#include "tms.h"
#include <math.h>

void pray(struct small_creature *c, long adjustment)
    {
    struct small_creature *ctarget;
    long which, difficulty, i, j, success, time, aborted, permanent;
    char message[256], temp[256];
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    success = 0;
    permanent = 0;
    which = prayer_parse();
    aborted = FALSE;
    if (!(c->extended == NULL))
	if (!(which == NONE))
	    {
	    time = time_parse();
	    time = min(time, 3L*HOUR);
	    if (!(time == NONE))
		{
		if (!(flag_check(prayers[which].flags, PRAY_AUTOMATIC)))
		    {
		    difficulty = prayers[which].difficulty;
		    if (!(flag_check(c->extended->faith->gifts,
		      prayers[which].gifts)))
			difficulty += ONE_NOTCH;
		    success = skill_check(c, c->extended->faith->faith +
		      c->extended->faith->prayers[which] +
		      status_check(c, STATUS_PRAYED) +
		      (long) log10((double) 1 + time/UNITS_PER_SECOND) +
		      adjustment, difficulty,
		      SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
		    permanent = FALSE;
		    if ((success >= 2) && (flag_check(prayers[which].flags,
		      PRAY_CRITICAL_SUCCESS_PERMANENT)))
			permanent = TRUE;
		    if ((success >= 3) && (flag_check(prayers[which].flags,
		      PRAY_TWO_CRITICAL_SUCCESSES_PERMANENT)))
			permanent = TRUE;
		    if ((success >= 4) && (flag_check(prayers[which].flags,
		      PRAY_THREE_CRITICAL_SUCCESSES_PERMANENT)))
			permanent = TRUE;
		    }
		else
		    success = 1;
		if (flag_check(prayers[which].flags, PRAY_REQUIRES_GIFT))
		    success &= flag_check(prayers[which].gifts,
		      c->extended->faith->gifts);
		switch(which)
		    {
		    case PRAYER_ADORATION:
			switch(trandom() % 6)
			    {
			    case 0:
				pmsg("\"God, you are amazing!\"");
			    break;
			    case 1:
				pmsg("\"What an awesome Artist would make \
such a Creation!\"");
			    break;
			    case 2:
				pmsg("\"God, you are so kind!\"");
			    break;
			    case 3:
				pmsg("\"God, I don't know how to express my \
awe  for you!\"");
			    break;
			    case 4:
				pmsg("\"God, you are holy.\"");
			    break;
			    case 5:
				pmsg("\"God, YOU ARE.\"");
			    break;
			    }
		    break;
		    case PRAYER_AEGIS_OF_GOD:
			pmsg("Lord, sheild me from harm.");
			if (success > 0)
			    status_add(c, STATUS_MIRACLE_AEGIS_OF_GOD,
			      success);
		    break;
		    case PRAYER_BLESSING:
			pmsg("Bless whom?");
			ctarget = loc_creature(c);
			if (ctarget != NULL)
			    {
			    if (success > 0)
				{
				status_add(ctarget, STATUS_MIRACLE_PROVIDENCE,
				  success);
				}
			    find_title(ctarget, temp);
			    switch(trandom() % 4)
				{
				case 0:
			    	    sprintf(message, "\"Lord, bless %s.\"",
				      temp);
				break;
				case 1:
				    sprintf(message, "\"Lord, fill %s with \
your blessing.\"", temp);
				break;
				case 2:
				    sprintf(message, "\"Lord, impart your \
blessing on %s.\"", temp);
				break;
				case 3:
				    sprintf(message, "\"Lord, bless %s richly.\
\"", temp);
				break;
				}
			    pmsg(message);
			    }
			else
			    aborted = TRUE;
		    break;
		    case PRAYER_BREATH_OF_GOD:
			pmsg("\"Every breath we breathe is the breath of \
God.");
			pmsg("God, thank you for the breath of life.\"");
		    break;
		    case PRAYER_BREATH_PRAYER:
			if (c->extended->faith->breath_prayer == NULL)
			    {
			    pmsg("Some people have a simple, one sentence \
prayer, which they try to pray with");
			    pmsg("each breath: a breath prayer.  What would \
you like for your breath prayer to be?");
			    c->extended->faith->breath_prayer =
			      (struct text_struct *) calloc(1, sizeof(struct
			      text_struct));
#ifdef DEBUG
			    c->extended->faith->breath_prayer->me =
			      c->extended->faith->breath_prayer;
#endif
			    if (tgets(c->extended->faith->breath_prayer->text))
				c->extended->faith->breath_prayer->next = NULL;
			    else
				{
				aborted = TRUE;
				free(c->extended->faith->breath_prayer);
				c->extended->faith->breath_prayer = NULL;
				}
			    }
			else
			    {
			    sprintf(message, "\"%s\"",
			      c->extended->faith->breath_prayer->text);
			    pmsg(message);
			    }
		    break;
		    case PRAYER_BREATHE_UNDERWATER:
			pmsg("\"Lord, enable me to breathe under water.\"");
			/* Eventually include followers.  */
			if (success > 0)
			    {
			    status_add(c, STATUS_BREATHE_WATER,
			      success);
			    if (permanent)
				add_permanent_status(c,
				  STATUS_BREATHE_WATER, 1);
			    }
		    break;
		    case PRAYER_CHANT:
			switch(trandom() % 4)
			    {
			    case 0:
				pmsg("Gloria Deo; pax terrae.");
			    break;
			    case 1:
				pmsg("Veni, veni, Emmanuel.");
			    break;
			    case 2:
				pmsg("Gloria in excelsis Deo.");
			    break;
			    case 3:
				pmsg("You chant praises to the Lord.");
			    break;
			    }
		    break;
		    case PRAYER_CHARIOT_OF_FIRE:
			pmsg("\"Lord, take me away in a chariot of fire.\"");
			if (success > 0)
			    chariot_of_fire(c);
		    break;
		    case PRAYER_CONTEMPLATE_SCRIPTURE:
			switch(trandom() % 6)
			    {
			    case 0:
				pmsg("Your word is a lamp to my feet and a \
light to my path.");
			    break;
			    case 1:
				pmsg("The greatest commandment is this: Hear, \
O Israel, the Lord your God is One.");
				pmsg("You shall love the Lord your God with \
all of your heart, and all of your soul,");
				pmsg("and all of your mind, and all of your \
might.");
				pmsg("And the second commandment is like it: \
You shall love your neighbor as yourself.");
			    break;
			    case 2:
				pmsg("Pray continually");
			    break;
			    case 3:
				pmsg("I give you a new commandment: that you \
love one another.  As I have loved you, so");
				pmsg("shall you love one another.");
			    break;
			    case 4:
				pmsg("There is no love like this: that a man\
lay down his life for his friends.");
			    break;
			    case 5:
				pmsg("Do unto others as you would have them \
do unto you.  This is the sum of the Law");
				pmsg("and the Prophets.");
			    break;
			    }
		    break;
		    case PRAYER_CONVERSE:
			if (success > 0)
			    status_add(c, STATUS_MIRACLE_CONVERSE, success);
			if (permanent)
			    add_permanent_status(c, STATUS_MIRACLE_CONVERSE, 1);
		    break;
		    case PRAYER_DANCE:
			switch(trandom() % 2)
			    {
			    case 0:
				pmsg("Dance, then, wherever you may be.");
				pmsg("For I am the Lord of the Dance, said \
He.");
			    break;
			    case 1:
				pmsg("If the Spirit of the Lord moves in my soul,");
				pmsg("Like David the victor I dance.");
			    break;
			    }
		    break;
		    case PRAYER_DAWN_OF_CREATION:
			if (success > 0)
			    {
			    switch(trandom() % 4)
				{
				case 0:
				    pmsg("You listen and hear the language from the Dawn of Creation, the language from");
				    nocap_pmsg("before the time when languages were shattered.");
				    morewait();
				    pmsg("The whole universe is ringing, with words that are breath and wind and song and");
				    nocap_pmsg("dance.");
				    morewait();
				    pmsg("And you can hear the Creator's voice, speaking power and peace and love.");
				    morewait();
				    pmsg("And you speak.");
				    morewait();
				    pmsg("And you listen.");
				    morewait();
				    pmsg("And you are loved.");
				break;
				}
			    }
			else
			    pmsg("You try to listen, but you cannot quite tell what.");
		    break;
		    case PRAYER_DIVINE_SUSTENANCE:
			pmsg("\"YOU ARE, sustain me with your very being.\"");
			if (success > 0)
			    {
			    status_add(c, STATUS_MIRACLE_DIVINE_SUSTENANCE,
			      success);
			    status_add(c, STATUS_HUNGRY, -100);
			    status_add(c, STATUS_THIRSTY, -100);
			    status_add(c, STATUS_TIRED, -100);
			    }
			if (permanent)
			    add_permanent_status(c,
			      STATUS_MIRACLE_DIVINE_SUSTENANCE, 1);
		    break;
		    case PRAYER_DOXOLOGY:
			pmsg("Praise God from whom all blessings flow.");
			pmsg("Praise Him all creatures here below.");
			pmsg("Praise Him above, ye Heavenly host.");
			pmsg("Praise Father, Son, and Holy Ghost.");
			pmsg("Amen.");
		    break;
		    case PRAYER_EMPTYING:
			switch(trandom() % 4)
			    {
			    case 0:
				pmsg("\"I empty myself before you, O Lord.\"");
			    break;
			    case 1:
				pmsg("God gives gifts where he finds hands \
empty enough to receive them.");
				pmsg("\tAugustine");
			    break;
			    case 2:
				pmsg("\"Here I am, Lord.  You there?\"");
			    break;
			    case 3:
				pmsg("\"Lord?\"");
			    break;
			    }
		    break;
		    case PRAYER_FAVOR_WITH_MAN:
			pmsg("\"Lord, grant me favor with mankind.\"");
			if (success > 0)
			    status_add(c, STATUS_MIRACLE_FAVOR_WITH_MAN,
			      success);
			if (permanent)
			    add_permanent_status(c,
			      STATUS_MIRACLE_FAVOR_WITH_MAN, 1);
		    break;
		    case PRAYER_FINAL_STRIKE:
			pmsg("\"Lord...  let it be now.\"");
			if (success > 0)
			    final_strike(c);
		    break;
		    case PRAYER_FIRE_FROM_HEAVEN:
			pmsg("\"Lord, let your consuming fire fall from \
Heaven.\"");
			if (success > 0)
			    if (!(fire_from_heaven(c, loc_creature(c), success)))
				aborted = TRUE;
		    break;
		    case PRAYER_FLYING:
			pmsg("\"Lord, let me fly.\"");
			if (success > 0)
			    status_add(c, STATUS_FLYING, success);
			if (permanent)
			    add_permanent_status(c, STATUS_FLYING, 1);
		    break;
		    case PRAYER_FORTITUDE:
			pmsg("\"Lord, give me courage.\"");
			if (success > 0)
			    {
			    flag_reset(&c->flags, AFRAID);
			    status_add(c, STATUS_MIRACLE_FORTITUDE, success);
			    }
			if (permanent)
			    add_permanent_status(c, STATUS_MIRACLE_FORTITUDE,
			      1);
		    break;
		    case PRAYER_FUNERAL:
			funeral(c, get_item(c, EXTRACT_RECURSIVE));
		    break;
		    case PRAYER_GUARDIAN_ANGEL:
			if (success > 0)
			    guardian_angel(c);
		    break;
		    case PRAYER_HEALING:
			pmsg("\"Lord, bring your healing.\"");
			if (success > 0)
			    if (!(healing(loc_creature(c), success)))
				aborted = TRUE;
		    break;
		    case PRAYER_HOLY_GROUND:
			if (success > 0)
			    holy_ground(c, success);
			if (permanent)
			    permanent_holy_ground(c);
		    break;
		    case PRAYER_HOLY_OF_HOLIES:
			pmsg("Let us therefore enter into the Holy of Holies \
by the blood of the Lamb.");
			if (flag_check(c->extended->faith->gifts,
			  GIFT_COMMUNION))
			    prayer_units(c, 2 * (1 + time/SECOND));
		    break;
		    case PRAYER_HUMBLING_BEFORE_GOD:
			switch(trandom() % 3)
			    {
			    case 0:
				pmsg("Humble yourself in the sight of the \
Lord,");
				pmsg("and He shall lift you up.");
			    break;
			    case 1:
				pmsg("\"Lord, I am nothing before you.\"");
			    break;
			    case 2:
				pmsg("\"Lord, I bow before you.\"");
			    break;
			    }
		    break;
		    case PRAYER_IMAGES:
			switch(trandom()%10)
			    {
			    case 0:
				pmsg("You close your eyes and imagine being \
in the serene blue deep.");
				morewait();
				pmsg("Dolphins play.");
				morewait();
				pmsg("The waves move back and forth, to and \
fro.");
				morewait();
				pmsg("You feel a deep peace.");
			    break;
			    case 1:
				pmsg("Colors spin and dance.  You sing a song \
of imagination.");
			    break;
			    case 2:
				pmsg("You imagine yourself walking through a \
deep, silent, peaceful forest.  And in");
				nocap_pmsg("every breeze, every flower, every \
twig, you hear the Creator's lovesong.");
			    break;
			    case 3:
				pmsg("The stars fly by in your imagination.");
				morewait();
				pmsg("You can almost hear their celestial \
music.");
			    break;
			    case 4:
				pmsg("You see yourself buried deep in the \
heart of the Father: loved, warmed, held.");
				morewait();
				pmsg("With every pulse, every beat, you can \
feel his love washing over, around, and");
				nocap_pmsg("inside you.");
			    break;
			    case 5:
				pmsg("You see a bright, shooting fire, a fire \
of love and energy beyond intense.");
				morewait();
				pmsg("It burns through you, and yet you are \
not consumed.");
				morewait();
				pmsg("You are filled with its strength.");
			    break;
			    case 6:
				pmsg("No images come.");
				morewait();
				pmsg("In the peace, in the silence, in the \
stillness,");
				nocap_pmsg("you sense that which lies far \
beyond any image or imagining.");
			    break;
			    case 7:
				pmsg("You see yourself in a forest, and all \
of the animals come around you,");
				nocap_pmsg("and lay down at your feet.");
			    break;
			    case 8:
				pmsg("Blue...  white...  red...  yellow...  \
green...");
				pmsg("Colors and notes fly about inside your \
imagination,");
				nocap_pmsg("and you see that they are all \
dancing the great Dance.");
			    break;
			    case 9:
				pmsg("You see pattern within pattern within \
pattern, hidden beauty within hidden");
				nocap_pmsg("hidden beauty.");
				morewait();
				pmsg("There is beautiful structure and \
design all about you.");
			    break;
			    }
		    break;
		    case PRAYER_INCREASE_ATTRIBUTE:
			pmsg("\"Lord, strengthen me.\"");
			if (success > 0)
			    increase_attribute(c, success);
		    break;
		    case PRAYER_INCREASE_FAITH:
			pmsg("\"Lord, increase my faith.\"");
			if (success > 0)
			    increase_faith(c, success);
		    break;
		    case PRAYER_INCREASE_SKILL:
			pmsg("\"Lord, help me to become more skilled.\"");
			if (success > 0)
			    increase_skill(c, success);
		    break;
		    case PRAYER_INSIGHT:
			pmsg("\"Lord, give me an insight.\"");
			if (success > 0)
			    hint();
		    break;
		    case PRAYER_INTERPRET_TONGUES:
			if (success > 0)
			    aborted = (!(interpret_tongues(c,
			      loc_creature(c))));
		    break;
		    case PRAYER_JANRA_BLESSING:
			pmsg("\"May the Lord bless and keep you,");
			pmsg("far away from us.\"");
		    break;
		    case PRAYER_KYTHE:
			switch(trandom() % 5)
			    {
			    case 0:
			    	pmsg("You close your eyes...");
				morewait();
				pmsg("Not to shut anything out,");
				morewait();
				pmsg("But to let God in.");
				morewait();
				pmsg("In a language beyond words, in a language beyond silence,");
				morewait();
				pmsg("You commune with the Creator.");
				morewait();
				pmsg("You pray with your body, your mind, your spirit.");
				morewait();
				pmsg("You speak, and what is more, you listen.");
				morewait();
				pmsg("Everything you hear, everything inside your spirit connects you with God.");
				morewait();
				pmsg("You are envelopped in the Father's love.");
			    break;
			    case 1:
				pmsg("Prose, when infused with life, becomes poetry.");
				pmsg("Poetry, when infused with life, becomes song.");
				pmsg("Song, when infused with life, becomes prayer.");
				morewait();
				pmsg("It is not so much that you pray, as that you are envelopped by a prayer that is");
				nocap_pmsg("a living thing, by a love that is more solid than rock and more radiant than");
				nocap_pmsg("the sun blazing in all of its glory.");
				morewait();
				pmsg("After a time, you open your eyes, and you see the divine light subtly shining");
				nocap_pmsg("through every object in sight.");
			    break;
			    case 2:
				pmsg("You are overwhelmed by a presence so awesome, so powerful, so mysterious, so");
				nocap_pmsg("real, that you are terrified.");
				morewait();
				pmsg("You are beyond awestruck.  Your heart is filled with a secret, a mystery, that");
				nocap_pmsg("you wonder for a moment why anyone thinks of anything else.");
				morewait();
				pmsg("The strength is not only flowing about you, but in you, strengthening you, so");
				nocap_pmsg("that you are not consumed, not destroyed.");
				morewait();
				pmsg("You suddenly see the question of belief in God very differently...");
				morewait();
				pmsg("God is present, and can now be even less denied than you yourself.");
				morewait();
				pmsg("You are slowly filled with peace.");
			    break;
			    case 3:
				pmsg("The Spirit envelops you, enfolds you, surrounds you.  You are drawn into the");
				nocap_pmsg("heart of the Father, and drawn nearer to every bird, every tree, every rock.");
				morewait();
				pmsg("Love over and around, love outside and within, love given and love received.");
				pmsg("As light illuminates the world before your eyes, love illuminates the world");
				nocap_pmsg("before your spirit.");
				morewait();
				pmsg("And it is beautiful.");
			    break;
			    case 4:
				pmsg("You pray with a language beyond language, in the root from which");
				nocap_pmsg("language and song alike have sprung.");
				morewait();
				pmsg("It is like an embrace, enfolding you.");
				morewait();
				pmsg("You are drawn deep into the heart of the Father,");
				morewait();
				nocap_pmsg("and there you rest.");
				morewait();
				pmsg("He loves you.");
				morewait();
				pmsg("He cares for you as his own dear child.");
				morewait();
				pmsg("You drink, and are filled.");
			    }
			flag_set(&game_variables->quests_completed, WAS);
		    break;
		    case PRAYER_LIGHTNING_STRIKE:
			if (success > 0)
			    if (!(lightning_strike(c, loc_creature(c), 
			      success)))
				aborted = TRUE;
		    break;
		    case PRAYER_LISTEN_TO_CREATION:
			pmsg("You grow still, and become attentive to the Creation all around you.");
			morewait();
			switch(trandom() % 4)
			    {
			    case 0:
				pmsg("You suddenly feel the things around you as a thousand love notes from the");
				pmsg("Creator, a thousand gifts of love.");
				morewait();
				pmsg("You are struck by them as if you were seeing them for the first time.");
			    break;
			    case 1:
				pmsg("Memories flow through your mind, of every beauty you have seen.  To that aged");
				nocap_pmsg("chorus are added the voices now around you: and you see, hear, and feel in");
				nocap_pmsg("everything around you a myriad of songs, forming one great Song in solid form.");
			    break;
			    case 2:
				pmsg("You see a cricket jumping by your feet, and you are struck by awe at what a");
				nocap_pmsg("wonderful piece of artwork, what an incredible machine, it is.");
			    break;
			    case 3:
			    	pmsg("\"Men go abroad to stare at the height of the mountains, wonder at the depths");
				nocap_pmsg("of the ocean, and yet pass themselves by without wondering.\"");
				nocap_pmsg("\t-Augustine");
				morewait();
				pmsg("You look in awe and wonder upon the body you were given, at the beauty and");
				nocap_pmsg("mystery which it contains.  Your body walks, and moves, and lives, and breathes.");
				morewait();
				nocap_pmsg("Your thumb alone is a marvel beyond the greatest of mortal craftsmen -- soft,");
				nocap_pmsg("strong, flexible and capable of motion, healing itself when it is wounded.  You");
				nocap_pmsg("have the strength, the power, the fortitude to grab hold of a rock and not be");
				nocap_pmsg("injured by its rough hardness, and yet the softness to touch the face of an");
				nocap_pmsg("infant, and draw a smile.");
				morewait();
				pmsg("You can only wonder at what an awesome Creator would make such a creation.");
			    break;
			    }
			if (flag_check(c->extended->faith->gifts,
			  GIFT_COMMUNION))
			    prayer_units(c, 2 * (1 + time/SECOND));
			flag_set(&game_variables->quests_completed, WAS);
		    break;
		    case PRAYER_LISTEN_TO_SILENCE:
			pmsg("You grow still.");
			morewait();
			switch(trandom() % 4)
			    {
			    case 0:
				pmsg("You can hear the silence, the \
stillness.");
				morewait();
				pmsg("You grow calm, and are set at rest.");
				morewait();
				pmsg("You abide in the presence of the \
Spirit.");
			    break;
			    case 1:
				pmsg("You hear an incredible beauty in the \
silence.");
				morewait();
				pmsg("It sounds as if you can almost hear \
catches of the Eternal Song.");
			    break;
			    case 2:
				pmsg("The silence is a kind of music.");
				morewait();
				pmsg("You hear the beautiful stillness which \
musicians adorn with notes,");
				nocap_pmsg("in all of its pureness and \
sparseness.");
			    break;
			    case 3:
				pmsg("The silence, and the One who abides in \
it, fills you and slowly builds in you");
				nocap_pmsg("a wholeness.");
				morewait();
				pmsg("Is this what Heaven is like?");
			    }
			if (flag_check(c->extended->faith->gifts,
			  GIFT_COMMUNION))
			    prayer_units(c, 2 * (1 + time/SECOND));
			flag_set(&game_variables->quests_completed, WAS);
		    break;
		    case PRAYER_LISTEN_TO_THE_ETERNAL_SONG:
			switch(trandom() % 4)
			    {
			    case 0:
				pmsg("Beyond sound, there is stillness.");
				pmsg("Beyond stillness, there is the Eternal Song.");
			    break;
			    case 1:
				pmsg("Every pore of your body tingles as you hear a deep, majestic, powerful song in");
				nocap_pmsg("the stillness.");
				morewait();
				pmsg("It is immanent.");
			    break;
			    case 2:
				pmsg("In every bit of light around you, you can almost hear the music of the Song.");
			    break;
			    case 3:
				pmsg("\"Fall on your knees,");
				pmsg("    O hear the angel voices.\"");
			    break;
			    }
			if (flag_check(c->extended->faith->gifts,
			  GIFT_COMMUNION))
			    prayer_units(c, 2 * (1 + time/SECOND));
			flag_set(&game_variables->quests_completed, WAS);
		    break;
		    case PRAYER_LITURGY:
			switch(trandom() % 4)
			    {
			    case 0:
				pmsg("\"Bless the seven peoples of the world: Nor'krin, Tuz, Urvanovestilli, Yedidia,");
				pmsg("Jec, Shal, Janra.");
			    break;
			    case 1:
				pmsg("\"Thank you for giving us spirit, soul, body.");
				morewait();
				pmsg("Thank you for placing us in the realm of Creation.\"");
			    break;
			    case 2:
				pmsg("\"Blessed are you, Lord God, Father and Creator, who reign over the whole cosmos.\"");
			    break;
			    case 3:
				pmsg("\"Thank you for making the Church a family of brothers and sisters, caring for");
				nocap_pmsg("each other.\"");
			    break;
			    }
			morewait();
			pmsg("Amen.");
		    break;
		    case PRAYER_LOVE:
			switch(trandom() % 4)
			    {
			    case 0:
				pmsg("\"I love you, Lord.\"");
			    break;
			    case 1:
				pmsg("\"God, you are my greatest love.\"");
			    break;
			    case 2:
				pmsg("\"Lord, I love you with my whole \
being.\"");
			    break;
			    case 3:
				pmsg("\"\"");
			    break;
			    }
		    break;
		    case PRAYER_MEDITATION:
			switch(trandom() % 6)
			    {
			    case 0:
				pmsg("\"I will not leave you as orphans; \
I will come to you.\"");
			    break;
			    case 1:
				pmsg("\"I am the vine; you are the \
branches.  Abide in me, and I will abide \
in you.\"");
			    break;
			    case 2:
				pmsg("\"The wind blows wherever it \
pleases.  You hear its sound, but you \
cannot");
				nocap_pmsg("tell where it is or where it is \
going.  So it is with everyone born of the");
				pmsg("Spirit.\"");
			    break;
			    case 3:
				pmsg("\"The time is coming and is now here \
when true worshipers will worship the \"");
				pmsg("Father in Spirit and in truth, for the \
Father seeks such worshipers.");
				pmsg("God is Spirit, and those who worship \
him must worship in Spirit and in Truth.\"");
			    break;
			    case 4:
				pmsg("\"I am the bread which came down from \
Heaven.\"");
			    break;
			    case 5:
				pmsg("\"Whoever drinks the water I give him \
will never thirst again.  Indeed, the \
water");
				pmsg("I give him will become in him a spring \
of water, welling up to eternal Life.");
			    break;
			    }
			if (flag_check(c->extended->faith->gifts,
			  GIFT_COMMUNION))
			    prayer_units(c, 2 * (1 + time/SECOND));
			flag_set(&game_variables->quests_completed, WAS);
		    break;
		    case PRAYER_NAME:
			/* As per Madelein L'Engle.  */
		    break;
		    case PRAYER_NAME_OF_JESUS:
			pmsg("\"Jesus\"");
		    break;
		    case PRAYER_OFFER_SELF:
			switch(trandom() % 4)
			    {
			    case 0:
				pmsg("\"I offer myself to you, Lord.\"");
			    break;
			    case 1:
				pmsg("\"Lord, I give you myself.\"");
			    break;
			    case 2:
				pmsg("\"Lord, take me as your child.\"");
			    break;
			    case 3:
				pmsg("\"Lord, I give you all that I have \
and all that I am.\"");
			    break;
			    }
		    break;
		    case PRAYER_PATERNOSTER:
			pmsg("\"Our Father,");
			pmsg("Who art in Heaven,");
			pmsg("Hallowed be Thy Name.");
			pmsg("Thy Kingdom come.");
			pmsg("Thy will be done,");
			pmsg("On earth as it is in Heaven.");
			pmsg("Give us this day our daily bread.");
			pmsg("Forgive us our debts,");
			pmsg("As we forgive our debtors.");
			pmsg("And lead us not into temptation,");
			pmsg("But deliver us from evil.");
			pmsg("For thine is the Kingdom, and the power, and \
he glory,");
			pmsg("Forever.");
			pmsg("Amen.");
		    break;
		    case PRAYER_PERSUASION:
			pmsg("Persuade whom or what?");
			ctarget = loc_creature(c);
			if (ctarget != NULL)
			    {
			    if (success > 0)
				{
				reset_hostility(ctarget, c);
				make_follow(ctarget, c);
				}
			    }
			else
			    {
			    pmsg("Aborted.");
			    aborted = TRUE;
			    }
		    break;
		    case PRAYER_PRAISE_GOD:
			switch(trandom() % 7)
			    {
			    case 0:
				pmsg("\"God, you are awesome.\"");
			    break;
			    case 1:
				pmsg("\"The whole earth declares your \
marvelous glory.\"");
			    break;
			    case 2:
				pmsg("\"Lord...  you are such an amazing \
being, as to have such a crazy generosity.\
\"");
			    break;
			    case 3:
				pmsg("\"What an Artist...  what an Artist.\"");
			    break;
			    case 4:
				pmsg("\"God, you are infinitely beyond \
anything I can ever say about you.\"");
			    break;
			    case 5:
				pmsg("\"The Universe is blessed indeed to \
have such a creator as you!\"");
			    break;
			    case 6:
				pmsg("\"You give us faith...  You alone are \
worthy of all of our faith and all of our");
				nocap_pmsg("trust.\"");
			    break;
			    }
		    break;
		    case PRAYER_PROLOGUE_TO_JOHN:
			pmsg("In the beginning was the Word,");
			pmsg("And the Word was with God,");
			pmsg("And the Word was God.");
			pmsg("He was with him in the beginning.");
			pmsg("Through him all things were made;");
			pmsg("Without him nothing was made which has been \
made.");
			pmsg("In him was Life,");
			pmsg("And that Life was the Light of men.");
			pmsg("And the Light shines in the darkness,");
			pmsg("And the darkness has not overcome it.");
			flag_set(&game_variables->quests_completed, WAS);
		    break;
		    case PRAYER_PROVIDENCE:
			if (success > 0)
			    status_add(c, STATUS_MIRACLE_PROVIDENCE, success);
		    break;
		    case PRAYER_RAISE_DEAD:
			if (success > 0)
			    {
			    pmsg("Corpse:");
			    raise_dead(c, get_item(c, EXTRACT_RECURSIVE),
			      success);
			    }
		    break;
		    case PRAYER_RECEIVE_GIFT:
			if (success > 0)
			    receive_gift(c, success);
		    break;
		    case PRAYER_REJUVENATE:
			ctarget = loc_creature(c);
			if (ctarget != NULL)
			    {
			    if (success > 0)
				rejuvenate(c, success);
			    }
			else
			    {
			    pmsg("Aborted.");
			    aborted = TRUE;
			    }
		    break;
		    case PRAYER_RESTORATION:
			ctarget = loc_creature(c);
			if (ctarget != NULL)
			    {
			    if (success > 0)
				restoration(c);
			    if (permanent)
				permanent_restore(c);
			    }
			else
			    {
			    pmsg("Aborted.");
			    aborted = TRUE;
			    }
		    break;
		    case PRAYER_REVEAL_TERRAIN:
			pmsg("\"Lord, show me how to get around.\"");
			if (success > 0)
			    {
			    for(i = 0; i < MAX_X; ++i)
				for(j = 0; j < MAX_Y; ++j)
				    flag_set(&current_level->space[i][j].flags,
				      KNOWN);
			    if (success > 1)
				for(i = 0; i < MAX_X; ++i)
				    for(j = 0; j < MAX_Y; ++j)
					if (flag_check(current_level->
					  space[i][j].flags, HAS_SECRET))
					    discover_secret(i, j);
			    update_window();
			    }
		    break;
		    case PRAYER_SANCTUARY:
			pmsg("\"Lord, make this place a sanctuary.\"");
			if (success > 0)
			    sanctuary(c);
			flag_set(&game_variables->quests_completed, WAS);
		    break;
		    case PRAYER_SHINING_LIGHT:
			pmsg("\"Lord, let your light shine.\"");
			if (success > 0)
			    shining_light(c);
		    break;
		    case PRAYER_SILENT_COMMUNION:
			switch(trandom() % 6)
			    {
			    case 0:
				pmsg("You enter God's presence.");
			    break;
			    case 1:
				pmsg("You become still.");
			    break;
			    case 2:
				pmsg("\"Be still, and know that I am God.\"");
			    break;
			    case 3:
				pmsg("You feel the presence of the Spirit.");
			    break;
			    case 4:
				pmsg("You feel a deep, silent peace.");
			    break;
			    case 5:
				pmsg("There is a joy bubbling from deep \
within you.");
			    break;
			    }
			if (flag_check(c->extended->faith->gifts,
			  GIFT_COMMUNION))
			    prayer_units(c, 2 * (1 + time/SECOND));
			flag_set(&game_variables->quests_completed, WAS);
		    break;
		    case PRAYER_SILENT_CONTEMPLATION:
			switch(trandom()%6)
			    {
			    case 0:
				pmsg("\"Let all mortal flesh keep silence...\"");
			    break;
			    case 1:
				pmsg("You are covered by a hushed silence as you contemplate the Lord God Almighty.");
			    break;
			    case 2:
				pmsg("\"Be still, and know that I am God.\"");
			    break;
			    case 3:
				pmsg("Your mind is filled with the greatness of God.");
			    break;
			    case 4:
				pmsg("You stand before God; his glory fills you.");
			    break;
			    case 5:
				pmsg("In the darkness, in the silence, you come before God.");
			    break;
			    }
			if (flag_check(c->extended->faith->gifts,
			  GIFT_COMMUNION))
			    prayer_units(c, 2 * (1 + time/SECOND));
			flag_set(&game_variables->quests_completed, WAS);
		    break;
		    case PRAYER_SING_IN_TONGUES:
			if (success > 0)
			    {
			    pmsg("The Spirit blows through you to sing \
praises to God, in unknown tongues.");
			    prayer_units(c, 2 * (1 + time/SECOND));
			    }
		    break;
		    case PRAYER_SING_PRAISES:
			switch(trandom() % 8)
			    {
			    case 0:
				pmsg("\"Sing Alleluia to the Lord.\"");
			    break;
			    case 1:
				pmsg("\"I will praise him!\"");
			    break;
			    case 2:
				while (trandom() % 2)
				    pmsg("\"Alleluia,\"");
				pmsg("\"Alleluia!\"");
			    break;
			    case 3:
				pmsg("\"Lord, you are more precious than \
silver.");
				pmsg("Lord, you are more costly than gold.");
				pmsg("Lord, you are more beautiful than \
diamonds,");
				pmsg("And nothing I desire compares to you.\"");
				if (trandom() % 3 == 0)
				    {
				    morewait();
				    pmsg("And the Father said...");
				    morewait();
				    pmsg("\"Child, you are more precious \
than silver.");
				    pmsg("Child, you are more costly than \
gold.");
				    pmsg("Child, you are more beautiful than \
diamonds,");
				    pmsg("And nothing I desire compares to \
you.\"");
				    }
			    break;
			    case 4:
				pmsg("\"Lord, in you I put my trust.");
				pmsg("Lord, I give you myself.\"");
			    break;
			    case 5:
				pmsg("\"Lord, I will always love you,");
				pmsg("And I will always serve you,");
				pmsg("And I will always glorify your Name.");
				morewait();
				pmsg("\"You are holy.");
				pmsg("You are love.");
				pmsg("You are Light.");
				morewait();
				pmsg("\"And I ask for nothing less than you.");
				pmsg("Heaven and Earth do not compare");
				pmsg("To you, O Lord.\"");
			    break;
			    case 6:
				pmsg("\"Your love is eternal.");
				pmsg("Your song is eternal.");
				pmsg("Make me eternal,");
				pmsg("My Lord.");
			    break;
			    case 7:
				pmsg("\"And I stand, I stand in awe of you,");
				pmsg("Holy God, to whom all praise is due,");
				pmsg("I stand in awe of you.\"");
			    break;
			    }
		    break;
		    case PRAYER_SLAY_CREATURE:
			ctarget = loc_creature(c);
			if (ctarget != NULL)
			    {
			    if (success > 0)
				{
				gain_experience(c, ctarget->epvalue);
				creature_death(ctarget);
				}
			    }
			else
			    {
			    pmsg("Aborted.");
			    aborted = TRUE;
			    }
		    break;
		    case PRAYER_SPEAK_IN_TONGUES:
			if (success > 0)
			    {
			    pmsg("The Spirit moves in you to praise God in \
unknown tongues.");
			    prayer_units(c, 2 * (1 + time/SECOND));
			    }
		    break;
		    case PRAYER_STAR_SONG:
			switch(trandom() % 3)
			    {
			    case 0:
				pmsg("This is my Father's world,");
				pmsg("And to my listening ears,");
				pmsg("All nature sings, and round me rings,");
				pmsg("The Music of the Spheres.");
			    break;
			    case 1:
				pmsg("Silent, powerful, of crystalline blue \
purity,");
				nocap_pmsg("the stars are singing, as they \
sang at the dawn of Creation.");
				morewait();
				pmsg("You feel as if you could almost reach \
out and touch one.");
			    break;
			    case 2:
				pmsg("You sit in silence, and listen to the \
song of the stars.");
				pmsg("It is cool, still, and infinitely \
subtle.");
				pmsg("It is like a glass of icy water.");
				if (trandom() % 2)
				    {
				    morewait();
				    pmsg("The sky is like a great cathedral, \
all of ice, with shining gems embedded");
				    nocap_pmsg("in the walls.");
				    morewait();
				    pmsg("Its coolness, its stillness, its \
beauty fills you.");
				    }
			    break;
			    }
			flag_set(&game_variables->quests_completed, WAS);
		    break;
		    case PRAYER_STILLNESS:
			switch(trandom() % 6)
			    {
			    case 0:
				pmsg("You grow still, and retreat into the Lord.");
			    break;
			    case 1:
				pmsg("You relax, to let your being be refreshed by the Being of Beings.");
			    break;
			    case 2:
				pmsg("You grow still.");
			    break;
			    case 3:
				pmsg("You grow still, and then find beyond the stillness an intense, blazing fire, a");
				nocap_pmsg("storm, that was always present in the stillness.");
			    break;
			    case 4:
				pmsg("As you grow still, you find yourself deep inside the Father's heart.");
			    break;
			    case 5:
				pmsg("\"You are my hiding place.\"");
			    break;
			    }
			flag_set(&game_variables->quests_completed, WAS);
		    break;
		    case PRAYER_STOP_TIME:
			pmsg("\"Lord, stop time around me.\"");
			if (success > 0)
			    stop_time(c, success);
		    break;
		    case PRAYER_SUBMISSION:
			switch(trandom() % 4)
			    {
			    case 0:
				pmsg("\"I submit to you, Lord.\"");
			    break;
			    case 1:
				pmsg("\"Not my will, but yours be done.\"");
			    break;
			    case 2:
				pmsg("\"Lord, I bow my will before you.\"");
			    break;
			    case 3:
				pmsg("\"Lord, I ask you to truly be my Lord and Master.\"");
			    break;
			    }
		    break;
		    case PRAYER_THANKSGIVING:
			switch(trandom() % 10)
			    {
			    case 0:
				pmsg("\"Thank you, Lord, for the breath of life.\"");
			    break;
			    case 1:
				pmsg("\"Thank you, Lord, for feeding me.\"");
			    break;
			    case 2:
				pmsg("\"Thank you, Lord, for the gift of song.\"");
			    break;
			    case 3:
				pmsg("\"Thank you, Lord, for the stars.\"");
			    break;
			    case 4:
				pmsg("\"Thank you, Lord, for giving us fire.\"");
			    break;
			    case 5:
				pmsg("\"Thank you, Lord, for my mind.\"");
			    break;
			    case 6:
				pmsg("\"Thank you, Lord, for my body.\"");
			    break;
			    case 7:
				pmsg("\"Thank you, Lord, for adventures.\"");
			    break;
			    case 8:
				pmsg("\"Thank you, Lord, for the gift of silence.\"");
			    break;
			    case 9:
				pmsg("\"Thank you, Lord, for everything.\"");
			    break;
			    }
		    break;
		    case PRAYER_TOGETHER_WITH_ANOTHER_PERSON:
			pmsg("Pray with whom?");
			ctarget = loc_creature(c);
			if (ctarget != NULL)
			    {
			    if (flag_check(ctarget->flags, HUMAN))
				pray(c, ONE_NOTCH);
			    else
				pmsg("You can only pray with humans.");
			    }
			else
			    aborted = TRUE;
		    break;
		    case PRAYER_TREMBLE:
			switch(trandom() % 4)
			    {
			    case 0:
				pmsg("You feel only the faintest hints of a presence,");
				morewait();
				nocap_pmsg("but it sets in your heart a deep awe and wonder.");
			    break;
			    case 1:
				pmsg("You feel deep within you a fear and amazement.");
				morewait();
				pmsg("How great is the cosmos, let alone its Creator!");
			    break;
			    case 2:
				pmsg("You quake in the presence of God.");
			    break;
			    case 3:
				pmsg("You catch for a moment a glimpse of the patterns in a tiny pebble, and your");
				nocap_pmsg("breath is taken away by the thought of what a Creator would fashion a whole");
				nocap_pmsg("cosmos, and yet put such detail into even a stone.");
			    break;
			    }
			flag_set(&game_variables->quests_completed, WAS);
			flag_set(&c->flags, AFRAID);
		    break;
		    case PRAYER_WALK_ON_WATER:
			pmsg("Lord, let me walk on water.");
			if (success > 0)
			    status_add(c, STATUS_MIRACLE_WALK_ON_WATER,
			      success);
		    break;
		    case PRAYER_WORSHIP:
			switch(trandom() % 4)
			    {
			    case 0:
				pmsg("You fall on your face before God.");
			    break;
			    case 1:
				pmsg("\"Lord, I worship you.\"");
			    break;
			    case 2:
				pmsg("\"Lord, YOU ARE.\"");
			    break;
			    case 3:
				pmsg("\"Lord, you are the apex of all that is good.\"");
			    break;
			    }
		    break;
		    case PRAYER_WORSHIP_SERVICE:
			pmsg("Worship with whom?");
			ctarget = loc_creature(c);
			if (ctarget != NULL)
			    {
			    if (flag_check(ctarget->flags, HUMAN))
				worship_service();
			    else
				{
				aborted = TRUE;
				pmsg("That creature is not human; aborting...");
				}
			    }
			else
			    {
			    aborted = 1;
			    pmsg("Aborted.");
			    }
		    break;
		    }
		}
	    else
		pmsg("Aborted.");
	    if (!(aborted))
		{
		take_time(c, time);

/*
 *
 * Barring critical failure, strengthen the creature's faith in that prayer.
 *
 */

		if (flag_check(c->extended->faith->gifts,
		  GIFT_WORD_OF_KNOWLEDGE))
		    {
		    if (success > 0)
			{
			if (success > 1)
			    pmsg("Granted in abundance.");
			else
			    pmsg("Granted.");
			}
		    else
			pmsg("Denied.");
		    }
		if (success > -1)
		    for(i = 0; i < success + 1; ++i)
			if (trandom() % (max(c->extended->faith->prayers[which], 1)) == 0)
			    ++c->extended->faith->prayers[which];

/*
 *
 * If we have a critical success, we strengthen the creature's overall faith.
 *
 */
		if (success > 1)
		    for (i = 0; i < success - 1; ++i)
			if (trandom() % (max(c->extended->faith->faith, 1)) == 0)
			    ++c->extended->faith->faith;
		prayer_units(c, 1 + time/SECOND);
		if (flag_check(c->extended->faith->gifts, GIFT_FAITH))
		    prayer_units(c, 1 + time/SECOND);
		morewait();
		}
	    else
		pmsg("Aborted.");
	    }
    }

void prayer_units(struct small_creature *c, long units)
    {
    long i;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c->extended != NULL)
	for(i = 0; i < units; ++i)
	    if (trandom() % (max(1, status_check(c, STATUS_PRAYED))) == 0)
		    status_add(c, STATUS_PRAYED, 1);
    }

long time_parse(void)
    {
    long number;
    char choice, player_input[256];;
    pmsg("How long: hours, minutes, or seconds (hms)?");
    choice = ' ';
    for( ; ((choice != 'h') && (choice != 'm') && (choice != 's') && (choice !=
      ESCAPE));choice = tgetc());
    switch(choice)
	{
	case 'h':
	    pmsg("How many hours?");
	break;
	case 'm':
	    pmsg("How many minutes?");
	break;
	case 's':
	    pmsg("How many seconds?");
	break;
	case ESCAPE:
	    return 0;
	break;
	}
    tgets(player_input);
    number = 0;
    sscanf(player_input, "%ld", &number);
    number = max(number, 0);
    switch(choice)
	{
	case 'h':
	    return number*HOUR;
	break;
	case 'm':
	    return number*MINUTE;
	break;
	case 's':
	    return number*SECOND;
	break;
	}
    return 0;
    }

void chariot_of_fire(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    read_file(FILE_CHARIOT_OF_FIRE);
    morewait();
    my_finish(0);
    }

void final_strike(struct small_creature *c)
    {
    long x, y;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    pmsg("The earth trembles, and you explode!");
    flag_reset(&game_variables->flags, PLAYER_AT_CENTER);
    while (trandom()%20)
	{
	switch(trandom()%4)
	    {
	    case 0: ++game_variables->display_center_x; break;
	    case 1: ++game_variables->display_center_y; break;
	    case 2: --game_variables->display_center_x; break;
	    case 3: --game_variables->display_center_y; break;
	    }
	update_window();
	}
    while (trandom()%1000)
	{
	x = trandom() % MAX_X;
	y = trandom() % MAX_Y;
	if ((clear_path(c->x, c->y, x, y)) && ((x != c->x) || (y != c->y)))
	    {
	    projectile_path(c->x, c->y, x, y, '*');
	    if (current_level->space[x][y].creatures != NULL)
		if (creature_damage(current_level->space[x][y].
		  creatures->current, 100, CREATURE_DAMAGE_WAKE_UP))
		    creature_death(current_level->space[x][y].creatures->
		      current);
	    }
	}
    morewait();
    creature_death(c);
    }

long fire_from_heaven(struct small_creature *c, struct small_creature *ctarget,
  long success)
    {
    char creature_title[256], message[256];
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
    check_pointer((struct pointer_struct *) ctarget);
#endif
    if (ctarget != NULL)
	{
	find_title(c, creature_title);
	sprintf(message, "Fire from Heaven descends on %s!", creature_title);
	if (can_see(c, ctarget->x, ctarget->y))
	    pmsg(message);
	if (creature_damage(ctarget, 10L * success, CREATURE_DAMAGE_WAKE_UP))
	    {
	    gain_ep(c, ctarget->epvalue);
	    creature_death(ctarget);
	    }
	return 1;
	}
    else
	return 0;
    }

void funeral(struct small_creature *c, struct small_item *i)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
    check_pointer((struct pointer_struct *) i);
#endif
    pmsg("\"Lord, let the deceased rest in peace.");
    morewait();
    pmsg("You burn the body.");
    if (c->extended != NULL)
	find_and_remove_item(i, &c->extended->pack);
    free_item(i);
    }

void guardian_angel(struct small_creature *c)
    {
    struct small_creature *guardian_angel;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    guardian_angel = guardian_angel_create();
    make_follow(guardian_angel, c);
    add_creature(current_level, guardian_angel, c->x, c->y);
    }

long healing(struct small_creature *c, long success)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c != NULL)
	{
	if (c->extended != NULL)
	    {
	    success = min(c->extended->stati->time[STATUS_WOUNDED], success);
	    status_add(c, STATUS_WOUNDED, 0 - success);
	    status_add(c, STATUS_HEALABLE, 0 - success);
	    status_add(c, STATUS_HEALABLE_ANTIDOTE, 0 - success);
	    calc_ready(c);
	    }
	else
	    {
	    success = min(c->wounded, success);
	    c->att += 3 * success;
	    c->def += 2 * success;
	    c->dmg += success;
	    c->hv += success;
	    c->sp += success;
	    c->wounded -= success;
	    }
	return TRUE;
	}
    else
	return FALSE;
    }

void holy_ground(struct small_creature *c, long success)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    flag_set(&current_level->space[c->x][c->y].flags, IS_SANCTUARY);
    flag_set(&current_level->space[c->x][c->y].flags, HOLY_GROUND);
    pmsg("You are standing on holy ground!");
    }

void increase_attribute(struct small_creature *c, long success)
    {
    long which_attribute;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    which_attribute = attribute_parse();
    if ((c->extended != NULL) && (which_attribute != NONE))
	{
	c->extended->attributes[which_attribute] += success;
	c->extended->perm_attributes[which_attribute] += success;
	calc_ready(c);
	}
    }

void increase_faith(struct small_creature *c, long success)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c->extended != NULL)
	if (c->extended->faith != NULL)
	    c->extended->faith->faith += success;
    }

void increase_skill(struct small_creature *c, long success)
    {
    long which_skill;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    which_skill = skill_parse();
    if ((c->extended != NULL) && (which_skill != NONE))
	{
	c->extended->skills->bv[which_skill] += success;
	c->extended->skills->av[which_skill] += success;
	check_skill_gain(c, which_skill, success);
	}
    }

long interpret_tongues(struct small_creature *interpreter, struct
  small_creature *speaker)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) interpreter);
    check_pointer((struct pointer_struct *) speaker);
#endif
    if ((interpreter != NULL) && (speaker != NULL))
	{
	if ((interpreter->extended != NULL) && (speaker->extended != NULL))
	    {
	    if ((interpreter->extended->faith != NULL) &&
	      (speaker->extended->faith != NULL))
		{
		if ((flag_check(interpreter->extended->faith->gifts,
		  GIFT_INTERPRETING_TONGUES)) && (flag_check(
		  speaker->extended->faith->gifts, GIFT_SPEAKING_IN_TONGUES)))
		    {
		    switch(trandom() % 12)
			{
			case 0:
			    pmsg("\"Holy is your name, O Lord.\"");
			break;
			case 1:
			    pmsg("\"God is love.\"");
			break;
			case 2:
			    pmsg("\"God has given us many gifts, from the food we eat to our very souls.  Let us");
			    nocap_pmsg("worship him.\"");
			break;
			case 3:
			    pmsg("\"Wonderful is the Lord, and greatly to be praised.\"");
			break;
			case 4:
			    pmsg("\"God is beyond all else.\"");
			break;
			case 5:
			    pmsg("\"God is the great Singer and Dancer who invites us all to become singers and");
			    nocap_pmsg("dancers with him.\"");
			break;
			case 6:
			    pmsg("\"God wants us to be his intimate children, to call him 'Daddy'.\"");
			break;
			case 7:
			    pmsg("\"Whatever can be said of God, even this, especially this, he is greater.\"");
			break;
			case 8:
			    pmsg("\"To God be all glory, forever!\"");
			break;
			case 9:
			    pmsg("\"Alleluia!\"");
			break;
			case 10:
			    pmsg("\"God loves to draw us into his heart.\"");
			break;
			case 11:
			    pmsg("\"We are imperfect; God is perfect.\"");
			break;
			}
		    return 1;
		    }
		else
		    return 0;
		}
	    else
		return 0;
	    }
	else
	    return 0;
	}
    else
	return 0;
    }

long lightning_strike(struct small_creature *c, struct small_creature *ctarget,
  long success)
    {
    char creature_title[256], message[256];
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
    check_pointer((struct pointer_struct *) ctarget);
#endif
    if (ctarget != NULL)
	{
	find_title(c, creature_title);
	sprintf(message, "Lightning strikes %s!", creature_title);
	if (can_see(c, ctarget->x, ctarget->y))
	    pmsg(message);
	if (creature_damage(ctarget, 10L * success, CREATURE_DAMAGE_WAKE_UP))
	    {
	    gain_ep(c, ctarget->epvalue);
	    creature_death(ctarget);
	    }
	return 1;
	}
    else
	return 0;
    }

void permanent_holy_ground(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    flag_set(&current_level->space[c->x][c->y].flags, HOLY_GROUND);
    flag_set(&current_level->space[c->x][c->y].flags, IS_SANCTUARY);
    pmsg("You are standing on holy ground!");
    }

void permanent_restore(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c->extended != NULL)
	{
	status_add(c, STATUS_COLD, -100);
	status_add(c, STATUS_FATIGUED, -100);
	status_add(c, STATUS_HUNGRY, -100);
	status_add(c, STATUS_PASSED_OUT, -100);
	status_add(c, STATUS_THIRSTY, -100);
	status_add(c, STATUS_TIRED, -100);
	}
    add_permanent_status(c, STATUS_MIRACLE_RESTORATION, 1);
    }

void raise_dead(struct small_creature *c, struct small_item *corpse, long
  success)
    {
    char title[256], message[256];
    struct small_creature *creature;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
    check_pointer((struct pointer_struct *) corpse);
#endif
    if (corpse != NULL)
	{
	if ((corpse->creatures != NULL) && (corpse->identity !=
	  MORTAL_REMAINS))
	    {
#ifdef DEBUG
	    if (corpse->creatures->next == NULL)
		pmsg("Bug pertaining to corpse; more than one creature.");
#endif
	    creature = corpse->creatures->current;
	    free_clist(corpse->creatures);
	    if (c->extended != NULL)
		{
		find_and_remove_item(corpse, &c->extended->pack);
		find_and_remove_item(corpse, &c->extended->wear_wield);
		}
	    find_title(creature, title);
	    sprintf(message, "%s is raised from the dead!", title);
	    pmsg(message);
	    if (success > 1)
		{
		restoration(creature);
		healing(creature, 5 * success);
		}
	    add_creature(current_level, creature, c->x, c->y);
	    }
	else
	    {
	    pmsg("That is not a corpse.");
	    }
	}
    }

void receive_gift(struct small_creature *c, long success)
    {
    long which, i;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    which = gift_parse();
    if (which != NONE)
	if (c->extended != NULL)
	    if (c->extended->faith != NULL)
		{
		flag_set(&c->extended->faith->gifts, which);
		for(i = 0; i < NUM_PRAYERS; ++i)
		    if (flag_check(prayers[i].gifts, which))
			while (trandom() % 10L * success != NULL)
			    ++c->extended->faith->prayers[i];
		}
    }

void rejuvenate(struct small_creature *c, long success)
    {
    long i, j, identity;
    char title[256], message[256];
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c->age > success)
	{
	find_title(c, title);
	if (c == player)
	    strcpy(message, "You grow younger.");
	else
	    sprintf(message, "%s grows younger.", title);
	pmsg(message);
	c->age -= success;
	if (c->identity < NUM_RACES)
	    identity = c->identity;
	else
	    identity = NUM_RACES;
	/* We don't start at zero, so that we can safely subtract one.  */
	for(i = 1; i < NUM_AGE_CLASSES; ++i)
	    if ((c->age < age_class_years[identity][i]) &&
	      (age_class_years[identity][i] <= c->age + success))
		{
		if (c->extended != NULL)
		    {
		    /* Rejuvenating can only improve attributes.  */
		    for(j = 0; j < NUM_ATTRIBUTES; ++j)
			{
			c->extended->attributes[j] += max(0, 
			  age_attribute_adjustment[i - 1][j] -
			  age_attribute_adjustment[i][j]);
			c->extended->perm_attributes[j] += max(0, 
			  age_attribute_adjustment[i - 1][j] -
			  age_attribute_adjustment[i][j]);
			}
		    calc_ready(c);
		    }
		}
	}
    }

void restoration(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c->extended != NULL)
	{
	status_add(c, STATUS_COLD, -100);
	status_add(c, STATUS_FATIGUED, -100);
	status_add(c, STATUS_HUNGRY, -100);
	status_add(c, STATUS_PASSED_OUT, -100);
	status_add(c, STATUS_THIRSTY, -100);
	status_add(c, STATUS_TIRED, -100);
	}
    }

void sanctuary(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    flag_set(&current_level->space[c->x][c->y].flags, IS_SANCTUARY);
    pmsg("You are standing on a sanctuary!");
    }

void shining_light(struct small_creature *c)
    {
    long i, j;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    pmsg("Beams of light illuminate the level!");
    for(i = 0; i < MAX_X; ++i)
	for(j = 0; j < MAX_Y; ++j)
	    {
	    flag_set(&current_level->space[i][j].flags, CURR_ILLUMINATED);
	    flag_set(&current_level->space[i][j].flags, PERM_ILLUMINATED);
	    }
    update_window();
    }

void stop_time(struct small_creature *c, long success)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    while(c->nextaction < game_variables->elapsed_units + success * MINUTE)
	get_command(c);
    c->nextaction = game_variables->elapsed_units;
    }
